#!/bin/bash
# ATMEL Wireless Card Setup for linux
# Copyright 2001 (c) ATMEL Hellas

if [ $# -eq 2 ]; then
	CONFIG_FILE=$1
        CONFIG_PCMCIA=$2
else
	echo "You must provide the output file and CONFIG_PCMCIA as args"
	exit 1
fi

function Checkxforms ()
{
	if [ -r /usr/include/X11/Xlib.h -o -r /usr/X11R6/include/X11/Xlib.h ]; then
		for f in /usr/{X11/,X11R6/,local/,}lib/libforms.{so,a} ; do
			if [ -r $f ]; then break; fi
		done
		for g in /usr/{X11/,X11R6/,local/,}include/{,,X11/}forms.h ; do
			if [ -r $g ]; then break; fi
		done
		if [ -r $f -a -r $g ] ; then
			HAS_FORMS=y
		else
			HAS_FORMS=n
		fi
	else
		echo "X Windows include files missing"
	fi
}

function CheckKernelVersion
{
    if [ -d $1 ]; then
        if [ -f $1/Makefile ]; then
            for TAG in VERSION PATCHLEVEL SUBLEVEL EXTRAVERSION ; do
                eval `sed -ne "/^$TAG/s/[ 	]//gp" $1/Makefile`
            done
            FV="$VERSION.$PATCHLEVEL.$SUBLEVEL$EXTRAVERSION"
            if [ "$LV" = "$FV" ]; then
                if [ "$PATH_FOUND" = "n" ]; then
                    PATH_FOUND=y
                    SRCPATH=$1
                fi
            fi
        fi
    fi
}


PATH_FOUND=n
HAS_FORMS=n
MAN_PATH=$(man -w ls|xargs dirname)
MPA=/usr/src
CONFIGP="/etc/pcmcia"
USB_CONFIG="/etc/hotplug"
LV=$(uname -r)

USBDES="/lib/modules/$LV/kernel/drivers/usb"
if [ ! -d $USBDES ]; then
	mkdir $USBDES
fi

if [ -d /lib/modules/$LV/pcmcia ] ; then
	PCMCIADES="/lib/modules/$LV/pcmcia"
elif [ -d /lib/modules/$LV/kernel/drivers/net/pcmcia ] ; then
	PCMCIADES="/lib/modules/$LV/kernel/drivers/net/pcmcia"
else 
	PCMCIADES="/lib/modules/$LV/pcmcia"
	mkdir $PCMCIADES
fi

PCIDES="/lib/modules/$LV/kernel/drivers/net"
MODPROBE_PATH=$(which modprobe)

Checkxforms

if [ $KERNEL_SRC ]; then 
	echo $KERNEL_SRC "(defined)"
else
	if [ -L /lib/modules/$LV/build ]&&[ -f /lib/modules/$LV/build/Makefile ]; then
		SRCPATH=/lib/modules/$LV/build
	else
		for f in $MPA/*; do CheckKernelVersion $f ; done
	fi

	echo "Kernel Version Running $LV"

	if [ "$SRCPATH" = "null" ]; then
		echo -e "Kernel Source Directory Not Found. Please give path : \c"
		read SRCPATH
		if [ ! -d $SRCPATH ]; then
			echo "Invalid Path"
			rm -f config.mk
		fi
	else
		echo Found Kernel Source Directory "($SRCPATH)"
	fi

	if [ `uname -r | cut -d'.' -f2` -gt "4" ]; then
		NEW_KERN=y

		if [ ! -f $SRCPATH/include/asm/irq_vectors.h ]; then
			ln -s $SRCPATH/include/asm/mach-default/irq_vectors.h  $SRCPATH/include/asm/irq_vectors.h
		fi
	else
		NEW_KERN=n
	fi

	echo KERNEL_SRC = $SRCPATH >> $CONFIG_FILE
fi

if [ "$CONFIG_PCMCIA" = "y" ]; then
    PCMCIA_IN_KERNEL=$(grep -c CONFIG_PCMCIA= $SRCPATH/.config)

    if [ $PCMCIA_IN_KERNEL -eq 1 ] ; then
        echo "using kernel pcmcia"
        PCMCIA_SRC=$SRCPATH
    else
        echo -n "root dir of external pcmcia: "
        read PCMCIA_SRC
    fi

    if [ ! -d $PCMCIA_SRC/include ]; then
        echo "pcmcia include dir $PCMCIA_SRC/include does not exist."
        exit 2;
    fi
else
    PCMCIA_SRC="PCMCIA was not configured"
fi

echo HAS_FORMS = $HAS_FORMS >> $CONFIG_FILE
if [ -f $PCMCIA_DES ];then
	echo PCMCIA_CONFIG = $CONFIGP >> $CONFIG_FILE
	echo PCMCIA_SRC = $PCMCIA_SRC >> $CONFIG_FILE
fi

echo PCI_DES = $PCIDES >> $CONFIG_FILE
if [ -f $PCMCIA_DES ];then
	echo PCMCIA_DES = $PCMCIADES >> $CONFIG_FILE
fi
echo USB_DES = $USBDES >> $CONFIG_FILE
echo USB_CONFIG = $USB_CONFIG >> $CONFIG_FILE
echo MAN_PATH = $MAN_PATH >> $CONFIG_FILE
echo NEW_KERN = $NEW_KERN >> $CONFIG_FILE
echo >> $CONFIG_FILE
echo >> $CONFIG_FILE
