/* ChoiceCtrl.h */
/*******************************************************************************************
	Copyright 2002-2003 ATMEL Corporation.

	This file is part of ATMEL Wireless LAN Drivers.

	ATMEL Wireless LAN Drivers is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	ATMEL Wireless LAN Drivers is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with ATMEL Wireless LAN Drivers; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*******************************************************************************************/

#ifndef __CHOICECTRL_H__
#define __CHOICECTRL_H__

#include <wx/wx.h>


class CAutoChoice : public wxChoice
{
public:
	CAutoChoice( wxWindow *parent, int id = -1, const wxPoint &pos = wxDefaultPosition, const wxSize &size = wxDefaultSize, int n = 0, wxString choices[ ] = NULL, long style = 0 );
	CAutoChoice( wxWindow *parent, int id, const wxPoint &pos, const wxSize &size, const wxArrayString &choices, long style = 0 );
	virtual ~CAutoChoice( );

public:
	bool SetStringSelection( const wxString &string );
	
public:
	void SetStrings( const wxString choices[], int n );
	void SetStrings( wxArrayString &choices );
};


class CStringHistory
{
protected:
	wxArrayString mStrings;
	int mMaxSize;

public:
	CStringHistory( int max_size = 10 );
	virtual ~CStringHistory( );

public:
	int GetMaxSize( );
	void SetMaxSize( int max_size );

public:
	bool Add( const wxString &entry );
	int GetCount( );
	void Clear( );

	wxString GetEntry( int index );

	wxString operator [ ]( int index );
};


class CHistoryCombo : public wxComboBox
{
protected:
	CStringHistory mHistory;
	int mHistoryAutoUpdateCounter;

	int mMaxLength;

public:
	CHistoryCombo( wxWindow *parent, int id = -1, const wxString &value = _( "" ), const wxPoint &pos = wxDefaultPosition, const wxSize &size = wxDefaultSize, int n = 0, const wxString choices[ ] = NULL, long style = 0 );
	virtual ~CHistoryCombo( );

public:
	void BeginHistoryChanges( );
	void AddHistory( const wxString &entry );
	void ClearHistory( );
	void EndHistoryChanges( );

	void SetHistoryMaxSize( int size );
	int GetHistorySize( );

	wxString GetHistoryEntry( int index );

public:
	void SetValueEx( const wxString &value );
	void SetMaxLength( int max_length );

protected:
	void refresh_history_list( );

protected:
	void OnText( wxCommandEvent &event );
};


class CRadioListBox : public wxCheckListBox
{
protected:

public:
	CRadioListBox( wxWindow *parent, int id = -1, const wxPoint &pos = wxDefaultPosition, const wxSize &size = wxDefaultSize, long style = 0 );
	virtual ~CRadioListBox( );

public:
	int GetChecked( );

protected:
	void OnToggle( wxCommandEvent &event );
};


#endif // __CHOICECTRL_H__


