/* Device.h */
/*******************************************************************************************
	Copyright 2002-2003 ATMEL Corporation.

	This file is part of ATMEL Wireless LAN Drivers.

	ATMEL Wireless LAN Drivers is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	ATMEL Wireless LAN Drivers is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with ATMEL Wireless LAN Drivers; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*******************************************************************************************/

#ifndef __DEVICE_H__
#define __DEVICE_H__

#include <wx/wx.h>
#include <wx/thread.h>
#include <wx/dynarray.h>
#include "WinterData.h"


class CSiteData;
class ArraySiteData;
class CWinterModel;


// Device classes section
///////////////////////////
class CVnetDeviceBase
// This is the base class for all classes describing a vnet device.
{
protected:
	wxCriticalSection mCsIoctl;
	wxString mUFName;
	wxString mDevName;
	CWinterModel *mModel;

public:
	CVnetDeviceBase( const wxString &name = _T( "Unknown" ) );
	virtual ~CVnetDeviceBase( );

public:
	virtual bool IsOk( );

	void SetUFName( const wxString &name );
	wxString GetUFName( );

	wxString GetDevName( );

	void AcquireLock( );
	void ReleaseLock( );

	CWinterModel *GetModel( );

public:
	virtual int GetAPInfo( CBssInfoData dest[ MAX_BSS_ENTRIES ] );
	virtual bool StartSiteSurvey( );
	virtual bool StopSiteSurvey( );
	virtual int GetSiteSurveyState( );
	virtual bool SelectAPByIndex( int index );

	// Later
	virtual bool GetDeviceConfig( CDeviceConfigurationData *dest );

	// Later
	virtual bool SetDeviceConfig( CDeviceConfigurationData *src );

	// Later
	virtual bool GetStatistics( CStatisticsData *data );

	// Later
	virtual bool GetMacAddress( CMacAddressData *data );

	// Get wep key information
	virtual bool GetWepInfo( CWepInfoData *data );
	
	// Set custom wep key values
	virtual bool SetWepInfo( CWepInfoData *data );

	// Gets driver, firmware versions for driver
	virtual bool GetVersionInfo( CVersionInfoData *data );

	// resets card. We could also use DS_RESET_CARD defined in pcmcia/ds.h.
	virtual bool ResetCard( );

	// clears packets
	virtual bool ClearPackets( );

	/*	For GUI	*/
	// Send Selected AP
	virtual bool SendSelectedAP( );
};


// Device tool classes section
////////////////////////////////
WX_DEFINE_ARRAY( CVnetDeviceBase *, ArrayVnetDevice );


class CVnetDeviceManager
{
protected:
	ArrayVnetDevice mDevices;

public:
	CVnetDeviceManager( );
	virtual ~CVnetDeviceManager( );

public:
	virtual bool Detect( );
	bool AddDevice( CVnetDeviceBase *device );

public:
	int GetDeviceCount( );
	CVnetDeviceBase *GetDevice( int index );
};


class CVnetDeviceManagerSingleton
{
protected:
	static CVnetDeviceManager *mManager;

protected:
	CVnetDeviceManagerSingleton( );

public:
	virtual ~CVnetDeviceManagerSingleton( );

public:
	static CVnetDeviceManager *Get( );
};


#endif // __DEVICE_H__


