/* EncryptionPage.cpp */
/*******************************************************************************************
	Copyright 2002-2003 ATMEL Corporation.

	This file is part of ATMEL Wireless LAN Drivers.

	ATMEL Wireless LAN Drivers is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	ATMEL Wireless LAN Drivers is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with ATMEL Wireless LAN Drivers; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*******************************************************************************************/

#include "EncryptionPage.h"
#include "Decls.h"
#include "ChoiceCtrl.h"
#include "WinterModel.h"
#include "WinterData.h"
#include "Accessor.h"
#include "WinterFrame.h"
#include "WinterTimer.h"
#include "HRStrings.h"
#include "Device.h"
#include "WinterSetup.h"
#include "WinterDebug.h"

#include <wx/busyinfo.h>


BEGIN_EVENT_TABLE( CEncryptionPage, wxPanel )
	EVT_BUTTON( ID_BT_ENCRYPTPAGE_SUBMIT, CEncryptionPage::OnBtEncryptPageSubmit )
	EVT_IDLE( CEncryptionPage::OnIdle )
END_EVENT_TABLE( )


CEncryptionPage::CEncryptionPage(  wxWindow *parent, int id, const wxPoint &pos, const wxSize &size, long style ) : CWinterPage( parent, id, pos )
{
	//EncryptionPage setup
	wxBoxSizer *sz_curline;

	int border = 4;
	wxSize page3_labelsize( 80, -1 );
	wxSize page3_labelsize_2( 150, -1 );
	int field = 6;
	int width = 20;

//First Box Controls
	//ID_SB_FIRSTBOX,
	mStEncryption = new wxStaticText( this, ID_ST_FIRSTBOX_ENCRYPTION, _( "Encryption :" ), wxDefaultPosition, page3_labelsize_2 );
	mChEncryption = new CAutoChoice( this, ID_CH_FIRSTBOX_ENCRYPTION );

//Second Box Controls
	//ID_SB_SECONDBOX,
	mStKey[0] = new wxStaticText( this, ID_ST_SECONDBOX_KEY1, _( "Key #1 " ), wxDefaultPosition, page3_labelsize );
	mTxKey[0] = new wxTextCtrl( this, ID_TX_SECONDBOX_KEY1, "", wxDefaultPosition, wxDefaultSize, wxTE_PASSWORD | wxTE_PROCESS_TAB );
	mStKey[1] = new wxStaticText( this, ID_ST_SECONDBOX_KEY2, _( "Key #2 " ), wxDefaultPosition, page3_labelsize );
	mTxKey[1] = new wxTextCtrl( this, ID_TX_SECONDBOX_KEY2, "", wxDefaultPosition, wxDefaultSize, wxTE_PASSWORD | wxTE_PROCESS_TAB );
	mStKey[2] = new wxStaticText( this, ID_ST_SECONDBOX_KEY3, _( "Key #3 " ), wxDefaultPosition, page3_labelsize );
	mTxKey[2] = new wxTextCtrl( this, ID_TX_SECONDBOX_KEY3, "", wxDefaultPosition, wxDefaultSize, wxTE_PASSWORD | wxTE_PROCESS_TAB );
	mStKey[3] = new wxStaticText( this, ID_ST_SECONDBOX_KEY4, _( "Key #4 " ), wxDefaultPosition, page3_labelsize );
	mTxKey[3] = new wxTextCtrl( this, ID_TX_SECONDBOX_KEY4, "", wxDefaultPosition, wxDefaultSize, wxTE_PASSWORD | wxTE_PROCESS_TAB );

//Third Box Controls
	//ID_SB_THIRDBOX,
	mStWepKeyToUse = new wxStaticText( this, ID_ST_THIRDBOX_WEPKEYTOUSE, _( "WEP Key to use " ), wxDefaultPosition, page3_labelsize_2 );
	mChWepkeyToUse = new CAutoChoice( this, ID_CH_THIRDBOX_WEPKEYTOUSE );
	mStWepMode = new wxStaticText( this, ID_ST_THIRDBOX_WEPMODE, _( "WEP Mode " ), wxDefaultPosition, page3_labelsize_2 );
	mChWepMode = new CAutoChoice( this, ID_CH_THIRDBOX_WEPMODE );
	mStAuthType =  new wxStaticText( this, ID_ST_THIRDBOX_AUTHENTICATIONTYPE, _( "Authentication Type " ), wxDefaultPosition, page3_labelsize_2 );
	mChAuthType = new CAutoChoice( this, ID_CH_THIRDBOX_AUTHENTICATIONTYPE );

//Encryption Page Controls
	mSubmit = new wxButton( this, ID_BT_ENCRYPTPAGE_SUBMIT, _( "&Submit" ), wxDefaultPosition, wxDefaultSize );

	wxStaticBox *sb_firstbox = new wxStaticBox( this, ID_SB_FIRSTBOX, wxT( "" ) );
	wxStaticBox *sb_secondbox = new wxStaticBox( this, ID_SB_SECONDBOX, wxT( "" ) );
	wxStaticBox *sb_thirdbox = new wxStaticBox( this, ID_SB_THIRDBOX, wxT( "" ) );

	//The first box group
	wxStaticBoxSizer *sz_firstbox = new wxStaticBoxSizer( sb_firstbox, wxVERTICAL );  

	//Encryption
	sz_curline = new wxBoxSizer( wxHORIZONTAL );
	sz_firstbox->Add( 1, 10, 0, 1 );
	sz_curline->Add( 1, 10, 1, 0 );
	sz_curline->Add( mStEncryption, 0, wxALIGN_CENTER_VERTICAL, 0 );
	sz_curline->Add( mChEncryption, field, wxALIGN_CENTER_VERTICAL, 0 );
	sz_curline->Add( width, 10, 0, 0 );
	sz_firstbox->Add( sz_curline, 0, wxEXPAND | wxALL, border );


	//The second box group
	wxStaticBoxSizer *sz_secondbox = new wxStaticBoxSizer( sb_secondbox, wxVERTICAL );

	//key #1
	sz_curline = new wxBoxSizer( wxHORIZONTAL );
	sz_curline->Add( 1, 10, 1, 0 );
	sz_curline->Add( mStKey[0], 0, wxALIGN_CENTER_VERTICAL, 0 );
	sz_curline->Add( mTxKey[0], field, wxALIGN_CENTER_VERTICAL, 0 );
	sz_curline->Add( width, 10, 0, 0 );
	sz_secondbox->Add( 1, 10, 0, 1 );
	sz_secondbox->Add( sz_curline, 0, wxEXPAND | wxALL, border );

	//key #2
	sz_curline = new wxBoxSizer( wxHORIZONTAL );
	sz_curline->Add( 1, 10, 1, 0 );
	sz_curline->Add( mStKey[1], 0, wxALIGN_CENTER_VERTICAL, 0 );
	sz_curline->Add( mTxKey[1], field, wxALIGN_CENTER_VERTICAL, 0 );
	sz_curline->Add( width, 10, 0, 0 );
	sz_secondbox->Add( sz_curline, 0, wxEXPAND | wxALL, border );

	//key #3

	sz_curline = new wxBoxSizer( wxHORIZONTAL );
	sz_curline->Add( 1, 10, 1, 0 );
	sz_curline->Add( mStKey[2], 0, wxALIGN_CENTER_VERTICAL, 0 );
	sz_curline->Add( mTxKey[2], field, wxALIGN_CENTER_VERTICAL, 0 );
	sz_curline->Add( width, 10, 0, 0 );
	sz_secondbox->Add( sz_curline, 0, wxEXPAND | wxALL, border );

	//key #4
	sz_curline = new wxBoxSizer( wxHORIZONTAL );
	sz_curline->Add( 1, 10, 1, 0 );
	sz_curline->Add( mStKey[3], 0, wxALIGN_CENTER_VERTICAL, 0 );
	sz_curline->Add( mTxKey[3], field, wxALIGN_CENTER_VERTICAL, 0 );
	sz_curline->Add( width, 10, 0, 0 );
	sz_secondbox->Add( sz_curline, 0, wxEXPAND | wxALL, border );

	//The third box group
	wxStaticBoxSizer *sz_thirdbox = new wxStaticBoxSizer( sb_thirdbox, wxVERTICAL );

	//wep key to use
	sz_curline = new wxBoxSizer( wxHORIZONTAL );
	sz_curline->Add( 1, 10, 1, 0 );
	sz_curline->Add( mStWepKeyToUse, 0, wxALIGN_CENTER_VERTICAL, 0 );
	sz_curline->Add( mChWepkeyToUse, field, wxALIGN_CENTER_VERTICAL, 0 );
	sz_curline->Add( width, 10, 0, 0 );
	sz_thirdbox->Add( 1, 10, 0, 1 );
	sz_thirdbox->Add( sz_curline, 0, wxEXPAND | wxALL, border );

	//wep mode
	sz_curline = new wxBoxSizer( wxHORIZONTAL );
	sz_curline->Add( 1, 10, 1, 0 );
	sz_curline->Add( mStWepMode, 0, wxALIGN_CENTER_VERTICAL, 0 );
	sz_curline->Add( mChWepMode, field, wxALIGN_CENTER_VERTICAL, 0 );
	sz_curline->Add( width, 10, 0, 0 );
	sz_thirdbox->Add( sz_curline, 0, wxEXPAND | wxALL, border );

	//authentication type
	sz_curline = new wxBoxSizer( wxHORIZONTAL );
	sz_curline->Add( 1, 10, 1, 0 );
	sz_curline->Add( mStAuthType, 0, wxALIGN_CENTER_VERTICAL, 0 );
	sz_curline->Add( mChAuthType, field, wxALIGN_CENTER_VERTICAL, 0 );
	sz_curline->Add( width, 10, 0, 0 );
	sz_thirdbox->Add( sz_curline, 0, wxEXPAND | wxALL, border );


	//encryption page sizer
	wxBoxSizer *sz_page_encryption = new wxBoxSizer( wxVERTICAL );
	sz_page_encryption->Add( sz_firstbox, 2, wxEXPAND | wxALL, 5 );
	sz_page_encryption->Add( sz_secondbox, 5, wxEXPAND | wxALL, 5 );
	sz_page_encryption->Add( sz_thirdbox, 4, wxEXPAND | wxALL, 5 );
	sz_page_encryption->Add( mSubmit, 0, wxALIGN_RIGHT | wxRIGHT, 5 );
	sz_page_encryption->Add( 1, 10, 1, 0 );
	SetSizer( sz_page_encryption );
	SetAutoLayout( true );


//////////////////////////////Encryption page choices  setup

	mEncryptionChoice = new CEncryptionChoiceSP( this );

	wxArrayString encryption_ret = mEncryptionChoice->ProvideStrings( ID_CH_FIRSTBOX_ENCRYPTION );
	mChEncryption->SetStrings( encryption_ret ); 

	wxArrayString wepkey_ret = mEncryptionChoice->ProvideStrings( ID_CH_THIRDBOX_WEPKEYTOUSE );
	mChWepkeyToUse->SetStrings( wepkey_ret );

	wxArrayString wepmode_ret = mEncryptionChoice->ProvideStrings( ID_CH_THIRDBOX_WEPMODE );
	mChWepMode->SetStrings( wepmode_ret );

	wxArrayString authentication_ret = mEncryptionChoice->ProvideStrings( ID_CH_THIRDBOX_AUTHENTICATIONTYPE );
	mChAuthType->SetStrings( authentication_ret ); 

	mLastKey = -1;
}


CEncryptionPage::~CEncryptionPage( )
{
	delete mEncryptionChoice;
}

void CEncryptionPage::OnBtEncryptPageSubmit( wxCommandEvent &event )
{
#if ( USE_BUSY_INFO == 1 )// Once again some more ifdef's!
	wxBusyInfo info( _( "Configuring device..." ) );
#endif // USE_BUSY_INFO == 1
	DEACTIVATE_TIMER( );
#if ( USE_BUSY_INFO == 0 )
	::wxBeginBusyCursor( );
#endif  // USE_BUSY_INFO == 0
	SubmitEncryptionPageData( );
#if ( DELAY_AFTER_SUBMIT == 1 )
	::wxSleep( 1 );
#endif // DELAY_AFTER_SUBMIT == 1
	UpdateWinterPage( );
#if ( USE_BUSY_INFO == 0 )
	::wxEndBusyCursor( );
#endif  // USE_BUSY_INFO == 0
	( ( wxNotebook * ) FindWindowById( ID_NB_MAINNOTEBOOK ) )->SetSelection( 0 );
	ACTIVATE_TIMER( );
	return;
}

void CEncryptionPage::SubmitEncryptionPageData( )
{
	TransferToModel( );
	CWinterModel *model = ( CWinterModel * ) CAccessor::Access( AID_CURRENTMODEL );
	CVnetDeviceBase *device = ( CVnetDeviceBase * ) CAccessor::Access( AID_CURRENTDEVICE );
	CWepInfoData *data = ( CWepInfoData * ) model->GetChild( MC_WEPINFODATA );

	if( device == NULL )
	{
		::wxMessageBox( _( "No device." ), wxT( "Winter" ), wxICON_HAND, ( CWinterFrame * ) CAccessor::Access( AID_FRAME ) );
		return;
	}

	if( device->SetWepInfo( data ) == false )
	{
		::wxMessageBox( _( "Failed to set encryption settings." ) );
		return;
	}

	return;
}

void CEncryptionPage::UpdateKeyControls( int which_encryption )
{
	bool enable_wepmode = true;

	switch( which_encryption )
	{
		case WEP_DISABLED:

			SetKeyToUse( NONE );

			mTxKey[0]->Clear( );
			mTxKey[1]->Clear( );
			mTxKey[2]->Clear( );
			mTxKey[3]->Clear( );
			enable_wepmode = false;
			break;

		case WEP_64bit:
			mTxKey[0]->SetMaxLength( SHORT_WEP_KEY_SIZE );
			mTxKey[1]->SetMaxLength( SHORT_WEP_KEY_SIZE );
			mTxKey[2]->SetMaxLength( SHORT_WEP_KEY_SIZE );
			mTxKey[3]->SetMaxLength( SHORT_WEP_KEY_SIZE );
			break;

		case WEP_128bit:
			mTxKey[0]->SetMaxLength( LONG_WEP_KEY_SIZE );
			mTxKey[1]->SetMaxLength( LONG_WEP_KEY_SIZE );
			mTxKey[2]->SetMaxLength( LONG_WEP_KEY_SIZE );
			mTxKey[3]->SetMaxLength( LONG_WEP_KEY_SIZE );
			break;
	}

	if( mChWepMode->IsEnabled( ) != enable_wepmode )
	{
		mChWepMode->Enable( enable_wepmode );
		mChAuthType->Enable( enable_wepmode );
		mChWepkeyToUse->Enable( enable_wepmode );
	}

	return;
}

void CEncryptionPage::OnIdle( wxIdleEvent &event )
{
	int current_key = mChEncryption->GetSelection( );

	if( mLastKey != current_key )
	{
		switch( mLastKey )
		{
			// Disabled
			case 0:
				break;
			// 64 bit
			// 128 bit
			case 1:
			case 2:
				mKeys[ mLastKey - 1 ][ 0 ] = mTxKey[0]->GetValue( );
				mKeys[ mLastKey - 1 ][ 1 ] = mTxKey[1]->GetValue( );
				mKeys[ mLastKey - 1 ][ 2 ] = mTxKey[2]->GetValue( );
				mKeys[ mLastKey - 1 ][ 3 ] = mTxKey[3]->GetValue( );
			default:
				break;
		}

		switch( current_key )
		{
			// Disabled
			case 0:
				mTxKey[0]->SetValue( _( "" ) );
				mTxKey[1]->SetValue( _( "" ) );
				mTxKey[2]->SetValue( _( "" ) );
				mTxKey[3]->SetValue( _( "" ) );
				break;
			// 64 bit
			// 128 bit
			case 1:
			case 2:
				mTxKey[0]->SetValue( mKeys[ current_key - 1 ][ 0 ] );
				mTxKey[1]->SetValue( mKeys[ current_key - 1 ][ 1 ] );
				mTxKey[2]->SetValue( mKeys[ current_key - 1 ][ 2 ] );
				mTxKey[3]->SetValue( mKeys[ current_key - 1 ][ 3 ] );
				break;
			default:
				break;
		}

		mLastKey = current_key;
		UpdateKeyControls( current_key );
	}

	if( mChWepkeyToUse->IsEnabled( ) )
	{

		SetKeyToUse( mChWepkeyToUse->GetSelection( ) + 1 );
	}
	else
	{
		SetKeyToUse( NONE );
	}

	event.Skip( );
	return; 
}

void CEncryptionPage::SetKeyToUse( int whichkey )
{
	switch( whichkey )
	{
		case NONE:
			mTxKey[0]->Enable( false );
			mTxKey[1]->Enable( false );
			mTxKey[2]->Enable( false );
			mTxKey[3]->Enable( false );
			break;

		case KEY1:
			mTxKey[0]->Enable( true );
			mTxKey[1]->Enable( false );
			mTxKey[2]->Enable( false );
			mTxKey[3]->Enable( false );
			break;

		case KEY2:
			mTxKey[0]->Enable( false );
			mTxKey[1]->Enable( true );
			mTxKey[2]->Enable( false );
			mTxKey[3]->Enable( false );
			break;

		case KEY3:
			mTxKey[0]->Enable( false );
			mTxKey[1]->Enable( false );
			mTxKey[2]->Enable( true );
			mTxKey[3]->Enable( false );
			break;

		case KEY4:
			mTxKey[0]->Enable( false );
			mTxKey[1]->Enable( false );
			mTxKey[2]->Enable( false );
			mTxKey[3]->Enable( true );
			break;
	}

	return;
}

// Higher update level
void CEncryptionPage::UpdateWinterPage( )
{
	CWinterModel *model = ( CWinterModel * ) CAccessor::Access( AID_CURRENTMODEL );
	CVnetDeviceBase *device = ( CVnetDeviceBase * ) CAccessor::Access( AID_CURRENTDEVICE );

	CWepInfoData *data = ( CWepInfoData * ) model->GetChild( MC_WEPINFODATA );
	if( device->GetWepInfo( data ) == false )
	{
		::wxMessageBox( _( "Failed to read WEP info." ), wxT( "Winter" ), wxICON_HAND, ( CWinterFrame * ) CAccessor::Access( AID_FRAME ) );
		return;
	}

	TransferToControls( );
	return;
}

int CEncryptionPage::TestControlValues( )
{
	return 0;
}

int CEncryptionPage::TransferToControls( CWinterModel *model )
{
	if( model == NULL )
	{
		model = ( CWinterModel * ) CAccessor::Access( AID_CURRENTMODEL );
	}

	CWepInfoData *wepinfo_data = ( CWepInfoData * ) model->GetChild( MC_WEPINFODATA );
	CWinterStringsProvider *sp = ( CWinterStringsProvider * ) CAccessor::Access( AID_STRINGSPROVIDER );


//EncryptionLevel
	mChEncryption->SetStringSelection( sp->EncryptionStrings[ ( int ) wepinfo_data->GetEncryptionLevel( ) ] );

//WepMode
	mChWepMode->SetStringSelection( sp->WepModeStrings[ ( int ) wepinfo_data->GetWepMode( ) ] );

//AuthType
	mChAuthType->SetStringSelection( sp->AuthTypeStrings[ ( int ) wepinfo_data->GetAuthenticationType( ) ] );

//WepKeysToUse
	if( ( int ) wepinfo_data->GetWepKeyToUse( ) != 0 )
	{
#ifdef ATMEL_WLAN
		mChWepkeyToUse->SetStringSelection( sp->WepKeysStrings[ ( int ) wepinfo_data->GetWepKeyToUse( ) ] );
#else
		mChWepkeyToUse->SetStringSelection( sp->WepKeysStrings[ ( int ) wepinfo_data->GetWepKeyToUse( ) - 1 ] );
#endif
	}
	else
	{
		// XXX: This code is kept so as to correct some differences between PCMCIA and USB devices. Hope these differences do not cause problems somewhere else in the code.
		mChWepkeyToUse->SetStringSelection( sp->WepKeysStrings[ 0 ] );
	}

//WepKeys
	UCHAR encryption = mChEncryption->GetSelection( );

	if( encryption == WEP_128bit )
	{
#ifdef ATMEL_WLAN
		UCHAR *key = new UCHAR[ LONG_WEP_KEY_SIZE + 1 ];
		for(int i=0; i<MAX_ENCRYPTION_KEYS; i++)
		{
			memset( key, 0, LONG_WEP_KEY_SIZE + 1 );
			memcpy( key, wepinfo_data->GetWepKey( i ), LONG_WEP_KEY_SIZE );
			mTxKey[i]->SetValue( wxString::Format( wxT( "%s" ), key ) );
		}
		delete key;
#else
		UCHAR *key1 = new UCHAR[ LONG_WEP_KEY_SIZE + 1 ];
		memset( key1, 0, LONG_WEP_KEY_SIZE + 1 );
		memcpy( key1, wepinfo_data->GetWepKey1( ), LONG_WEP_KEY_SIZE );
		mTxKey[0]->SetValue( wxString::Format( wxT( "%s" ), key1 ) );
		delete key1;

		UCHAR *key2 = new UCHAR[ LONG_WEP_KEY_SIZE + 1 ];
		memset( key2, 0, LONG_WEP_KEY_SIZE + 1 );
		memcpy( key2, wepinfo_data->GetWepKey2( ), LONG_WEP_KEY_SIZE );        
		mTxKey[1]->SetValue( wxString::Format( wxT( "%s" ), key2 ) );
		delete key2;

		UCHAR *key3 = new UCHAR[ LONG_WEP_KEY_SIZE + 1 ];
		memset( key3, 0, LONG_WEP_KEY_SIZE+ 1 );
		memcpy( key3, wepinfo_data->GetWepKey3( ), LONG_WEP_KEY_SIZE );        
		mTxKey[2]->SetValue( wxString::Format( wxT( "%s" ), key3 ) );
		delete key3;

		UCHAR *key4 = new UCHAR[ LONG_WEP_KEY_SIZE + 1 ];
		memset( key4, 0, LONG_WEP_KEY_SIZE+ 1 );
		memcpy( key4, wepinfo_data->GetWepKey4( ), LONG_WEP_KEY_SIZE );        
		mTxKey[3]->SetValue( wxString::Format( wxT( "%s" ), key4 ) );
		delete key4;
#endif
	}
	if( encryption == WEP_64bit )
	{
#ifdef ATMEL_WLAN
		UCHAR *key = new UCHAR[ SHORT_WEP_KEY_SIZE + 1 ];
		for(int i=0; i<MAX_ENCRYPTION_KEYS; i++)
		{
			memset( key, 0, SHORT_WEP_KEY_SIZE + 1 );
			memcpy( key, wepinfo_data->GetWepKey( i ), SHORT_WEP_KEY_SIZE );
			mTxKey[i]->SetValue( wxString::Format( wxT( "%s" ), key ) );
		}
		delete key;
#else
		UCHAR *key1 = new UCHAR[ SHORT_WEP_KEY_SIZE + 1 ];
		memset( key1, 0, SHORT_WEP_KEY_SIZE + 1 );
		memcpy( key1, wepinfo_data->GetWepKey1( ), SHORT_WEP_KEY_SIZE );        
		mTxKey[0]->SetValue( wxString::Format( wxT( "%s" ), key1 ) );
		delete key1;

		UCHAR *key2 = new UCHAR[ SHORT_WEP_KEY_SIZE + 1 ];
		memset( key2, 0, SHORT_WEP_KEY_SIZE + 1 );
		memcpy( key2, wepinfo_data->GetWepKey2( ), SHORT_WEP_KEY_SIZE );        
		mTxKey[1]->SetValue( wxString::Format( wxT( "%s" ), key2 ) );
		delete key2;

		UCHAR *key3 = new UCHAR[ SHORT_WEP_KEY_SIZE + 1 ];
		memset( key3, 0, SHORT_WEP_KEY_SIZE + 1 );
		memcpy( key3, wepinfo_data->GetWepKey3( ), SHORT_WEP_KEY_SIZE );        
		mTxKey[2]->SetValue( wxString::Format( wxT( "%s" ), key3 ) );
		delete key3;

		UCHAR *key4 = new UCHAR[ SHORT_WEP_KEY_SIZE + 1 ];
		memset( key4, 0, SHORT_WEP_KEY_SIZE + 1 );
		memcpy( key4, wepinfo_data->GetWepKey4( ), SHORT_WEP_KEY_SIZE );        
		mTxKey[3]->SetValue( wxString::Format( wxT( "%s" ), key4 ) );
		delete key4;
#endif	// ATMEL_WLAN
	}

	return 0;
}

int CEncryptionPage::TransferToModel( CWinterModel *model )
{
	if( model == NULL )
	{
		model = ( CWinterModel * ) CAccessor::Access( AID_CURRENTMODEL );
	}

	CWepInfoData *wepinfo_data = ( CWepInfoData * ) model->GetChild( MC_WEPINFODATA );
	CWinterStringsProvider *sp = ( CWinterStringsProvider * ) CAccessor::Access( AID_STRINGSPROVIDER );


//EncryptionLevel
	wepinfo_data->SetEncryptionLevel( sp->EncryptionStrings.Index( mChEncryption->GetStringSelection( ) ) );

//WepMode
	wepinfo_data->SetWepMode( sp->WepModeStrings.Index( mChWepMode->GetStringSelection( ) ) );

//AuthType
	wepinfo_data->SetAuthenticationType( sp->AuthTypeStrings.Index( mChAuthType->GetStringSelection( ) ) );

//WepKeysToUse
#ifndef ATMEL_WLAN
	wepinfo_data->SetWepKeyToUse( sp->WepKeysStrings.Index( mChWepkeyToUse->GetStringSelection( ) ) + 1 );
#else
	wepinfo_data->SetWepKeyToUse( sp->WepKeysStrings.Index( mChWepkeyToUse->GetStringSelection( ) ) );
#endif

//WepKeys
	UCHAR encryption = mChEncryption->GetSelection( );

	if( encryption == WEP_128bit )
	{
#ifdef ATMEL_WLAN
		UCHAR *key = new UCHAR[ LONG_WEP_KEY_SIZE + 1 ];
		for(int i=0; i<MAX_ENCRYPTION_KEYS; i++)
		{
			memset( key, 0, LONG_WEP_KEY_SIZE + 1 );
			wxString tx_key = mTxKey[i]->GetValue( );
			strcpy( ( char * ) key, tx_key.c_str( ) );
			wepinfo_data->SetWepKey( key, i );
		}
		delete key;
#else
		UCHAR *key1 = new UCHAR[ LONG_WEP_KEY_SIZE + 1 ];
		memset( key1, 0, LONG_WEP_KEY_SIZE + 1 );
		wxString tx_key1 = mTxKey[0]->GetValue( );
		strcpy( ( char * ) key1, tx_key1.c_str( ) );
		wepinfo_data->SetWepKey1( key1 );
		delete key1;

		UCHAR *key2 = new UCHAR[ LONG_WEP_KEY_SIZE + 1 ];
		memset( key2, 0, LONG_WEP_KEY_SIZE+ 1 );
		wxString tx_key2 = mTxKey[1]->GetValue( );
		strcpy( ( char * ) key2, tx_key2.c_str( ) );
		wepinfo_data->SetWepKey2( key2 );
		delete key2;

		UCHAR *key3 = new UCHAR[ LONG_WEP_KEY_SIZE + 1 ];
		memset( key3, 0, LONG_WEP_KEY_SIZE + 1 );
		wxString tx_key3 = mTxKey[2]->GetValue( );
		strcpy( ( char * ) key3, tx_key3.c_str( ) );
		wepinfo_data->SetWepKey3( key3 );
		delete key3;

		UCHAR *key4 = new UCHAR[ LONG_WEP_KEY_SIZE ];
		memset( key4, 0, LONG_WEP_KEY_SIZE + 1 );
		wxString tx_key4 = mTxKey[3]->GetValue( );
		strcpy( ( char * ) key4, tx_key4.c_str( ) );
		wepinfo_data->SetWepKey4( key4 );
		delete key4;
#endif
	}

	if( encryption == WEP_64bit )
	{
#ifdef ATMEL_WLAN
		UCHAR *key = new UCHAR[ SHORT_WEP_KEY_SIZE + 1 ];
		for(int i=0; i<MAX_ENCRYPTION_KEYS; i++)
		{
			memset( key, 0, SHORT_WEP_KEY_SIZE + 1 );
			wxString tx_key = mTxKey[i]->GetValue( );
			strcpy( ( char * ) key, tx_key.c_str( ) );
			wepinfo_data->SetWepKey( key, i );
		}
		delete key;
#else
		UCHAR *key1 = new UCHAR[ SHORT_WEP_KEY_SIZE + 1 ];
		memset( key1, 0, SHORT_WEP_KEY_SIZE + 1 );
		wxString tx_key1 = mTxKey[0]->GetValue( );
		strcpy( ( char * ) key1, tx_key1.c_str( ) );
		wepinfo_data->SetWepKey1( key1 );
		delete key1;

		UCHAR *key2 = new UCHAR[ SHORT_WEP_KEY_SIZE + 1 ];
		memset( key2, 0, SHORT_WEP_KEY_SIZE + 1 );
		wxString tx_key2 = mTxKey[1]->GetValue( );
		strcpy( ( char * ) key2, tx_key2.c_str( ) );
		wepinfo_data->SetWepKey2( key2 );
		delete key2;

		UCHAR *key3 = new UCHAR[ SHORT_WEP_KEY_SIZE + 1 ];
		memset( key3, 0, SHORT_WEP_KEY_SIZE + 1 );
		wxString tx_key3 = mTxKey[2]->GetValue( );
		strcpy( ( char * ) key3, tx_key3.c_str( ) );
		wepinfo_data->SetWepKey3( key3 );
		delete key3;

		UCHAR *key4 = new UCHAR[ SHORT_WEP_KEY_SIZE + 1 ];
		memset( key4, 0, SHORT_WEP_KEY_SIZE + 1 );
		wxString tx_key4 = mTxKey[3]->GetValue( );
		strcpy( ( char * ) key4, tx_key4.c_str( ) );
		wepinfo_data->SetWepKey4( key4 );
		delete key4;
#endif
	}
	return 0;
}


