/* MonitorPage.cpp */
/*******************************************************************************************
	Copyright 2002-2003 ATMEL Corporation.

	This file is part of ATMEL Wireless LAN Drivers.

	ATMEL Wireless LAN Drivers is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	ATMEL Wireless LAN Drivers is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with ATMEL Wireless LAN Drivers; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*******************************************************************************************/

#include "MonitorPage.h"
#include "ChoiceCtrl.h"
#include "Decls.h"
#include "Accessor.h"
#include "WinterFrame.h"
#include "WinterTimer.h"
#include "WinterDebug.h"
#include "HRStrings.h"
#include "Device.h"
#include "WinterSetup.h"

#include <stdlib.h>
#include <wx/busyinfo.h>


BEGIN_EVENT_TABLE( CMonitorPage, wxPanel )
	EVT_BUTTON( ID_BT_MONITORPAGE_CHANGE, CMonitorPage::OnBtChange )
	EVT_BUTTON( ID_BT_MONITORPAGE_SUBMIT, CMonitorPage::OnBtSubmit )
	EVT_BUTTON( ID_BT_MONITORPAGE_CANCEL, CMonitorPage::OnBtCancel )
	EVT_CHOICE( ID_CH_INROAMING, CMonitorPage::OnChIntRoamingChanged )
	EVT_IDLE( CMonitorPage::OnIdle )
END_EVENT_TABLE( )


CMonitorPage::CMonitorPage( wxWindow *parent, int id, const wxPoint &pos, const wxSize &size, long style ) : CWinterPage( parent, id, pos )
{
	mIsDirty = false;
	mInChangeMode = false;
	mStationState = -1;

	// Page setup
	wxSize page1_labelsize( 130, -1 );
	int bt_w = 80; // This is the width of the buttons.
	int field_weight = 3;
	int sizer_border = 15;

	wxBoxSizer *sz_currentline;

	wxStaticBox *sb_settings = new wxStaticBox( this, ID_SB_SETTINGS, _( "Settings" ) );
	wxStaticBox *sb_monitor = new wxStaticBox( this, ID_SB_MONITOR, _( "Monitor" ) );

	// The settings control group
	wxStaticBoxSizer *sz_settings = new wxStaticBoxSizer( sb_settings, wxVERTICAL );

	// Operating mode
	sz_currentline = new wxBoxSizer( wxHORIZONTAL );
	sz_currentline->Add( new wxStaticText( this, ID_ST_OPERATINGMODE, _( "Operating Mode :" ), wxDefaultPosition, page1_labelsize ), 0, wxALIGN_CENTER_VERTICAL, 0 );
	sz_currentline->Add(  new CAutoChoice( this, ID_CH_OPERATINGMODE ), field_weight, wxALIGN_CENTER_VERTICAL, 0 );
	sz_currentline->Add( 1, 10, 1, 0 );
	sz_currentline->Add( bt_w, 10, 0, 0 );
	sz_settings->Add( sz_currentline, 0, wxEXPAND | wxALL, sizer_border );

	// Channel   
	sz_currentline = new wxBoxSizer( wxHORIZONTAL );
	sz_currentline->Add( new wxStaticText( this, ID_ST_CHANNEL, _( "Channel : " ), wxDefaultPosition, page1_labelsize ), 0, wxALIGN_CENTER_VERTICAL, 0 );
	sz_currentline->Add( new CAutoChoice( this, ID_CH_CHANNEL ), field_weight, wxALIGN_CENTER_VERTICAL, 0 );
	sz_currentline->Add( 1, 10, 1, 0 );
	sz_currentline->Add( new wxButton( this, ID_BT_MONITORPAGE_CHANGE, _( "C&hange" ), wxDefaultPosition, wxSize( bt_w, -1 ) ), 0, wxALIGN_RIGHT, 0 );
	sz_settings->Add( sz_currentline, 0, wxEXPAND | wxLEFT | wxRIGHT | wxBOTTOM, sizer_border );

	// ESSID
	sz_currentline = new wxBoxSizer( wxHORIZONTAL );
	sz_currentline->Add( new wxStaticText( this, ID_ST_ESSID, wxT( "ESSID : " ), wxDefaultPosition, page1_labelsize ), 0, wxALIGN_CENTER_VERTICAL, 0 );
	sz_currentline->Add( new CHistoryCombo( this, ID_CB_ESSID, "", wxDefaultPosition, wxDefaultSize , 0, NULL, wxCB_DROPDOWN ), field_weight, wxALIGN_CENTER_VERTICAL, 0 );
	sz_currentline->Add( 1, 10, 1, 0 );
	sz_currentline->Add( new wxButton( this, ID_BT_MONITORPAGE_SUBMIT, _( "&Submit" ), wxDefaultPosition, wxSize( bt_w, -1 ) ),0,wxALIGN_CENTER_VERTICAL,0 );
	sz_settings->Add( sz_currentline, 0, wxEXPAND | wxLEFT | wxRIGHT | wxBOTTOM, sizer_border );

	// Radio on/off
	sz_currentline = new wxBoxSizer( wxHORIZONTAL );
	sz_currentline->Add( new wxStaticText( this, ID_ST_RADIOONOFF, _( "Radio On/Off :" ), wxDefaultPosition, page1_labelsize ), 0, wxALIGN_CENTER_VERTICAL, 0 );
	sz_currentline->Add( new CAutoChoice( this, ID_CH_RADIOONOFF ), field_weight, wxALIGN_CENTER_VERTICAL, 0 );
	sz_currentline->Add( 1, 10, 1, 0 );
	sz_currentline->Add( new wxButton( this, ID_BT_MONITORPAGE_CANCEL, _( "&Cancel" ), wxDefaultPosition, wxSize( bt_w, -1 ) ), 0, wxALIGN_CENTER_VERTICAL,0 );
	sz_settings->Add( sz_currentline, 0, wxEXPAND | wxLEFT | wxRIGHT | wxBOTTOM, sizer_border );

	// Int Roaming
	sz_currentline = new wxBoxSizer( wxHORIZONTAL );
	sz_currentline->Add( new wxStaticText( this, ID_ST_INROAMING, _( "Int. Roaming :" ), wxDefaultPosition, page1_labelsize ), 0, wxALIGN_CENTER_VERTICAL, 0 );
	sz_currentline->Add( new CAutoChoice( this, ID_CH_INROAMING ), field_weight, wxALIGN_CENTER_VERTICAL, 0 );
	sz_currentline->Add( 1, 10, 1, 0 );
	sz_currentline->Add( bt_w, 10, 0, 0 );
	sz_settings->Add( sz_currentline, 0, wxEXPAND | wxLEFT | wxRIGHT | wxBOTTOM, sizer_border );

	// Transmit rate
	sz_currentline = new wxBoxSizer( wxHORIZONTAL );
	sz_currentline->Add( new wxStaticText( this, ID_ST_TRANSMITRATE, _( "Transmit Rate :" ), wxDefaultPosition, page1_labelsize ), 0, wxALIGN_CENTER_VERTICAL, 0 );
	sz_currentline->Add( new CAutoChoice( this, ID_CH_TRANSMITRATE ), field_weight, wxALIGN_CENTER_VERTICAL, 0 );
	sz_currentline->Add( 1, 10, 1, 0 );
	sz_currentline->Add( bt_w, 10, 0, 0 );
	sz_settings->Add( sz_currentline, 0, wxEXPAND | wxLEFT | wxRIGHT | wxBOTTOM, sizer_border );

	// MAC address
	sz_currentline = new wxBoxSizer( wxHORIZONTAL );
	sz_currentline->Add( new wxStaticText( this, ID_ST_MACADDRESS, _( "Mac Address : " ), wxDefaultPosition, page1_labelsize ), 0, wxALIGN_CENTER_VERTICAL, 0 );
	sz_currentline->Add( new wxTextCtrl( this, ID_TX_MACADDRESS, _( "" ), wxDefaultPosition, wxDefaultSize, wxTE_READONLY ), field_weight, wxALIGN_CENTER_VERTICAL, 0 );
	sz_currentline->Add( 1, 10, 1, 0 );
	sz_currentline->Add( bt_w, 10, 0, 0 );
	sz_settings->Add( sz_currentline, 0, wxEXPAND | wxLEFT | wxRIGHT | wxBOTTOM, sizer_border );

	// The monitor group
	wxStaticBoxSizer *sz_monitor = new wxStaticBoxSizer( sb_monitor, wxVERTICAL );

	// Status
	sz_currentline = new wxBoxSizer( wxHORIZONTAL );
	sz_monitor->Add( 1, 10, 0, 1 );
	sz_currentline->Add( new wxStaticText( this, ID_ST_STATUS, _( "Status : " ), wxDefaultPosition, page1_labelsize ), 0, wxALIGN_CENTER_VERTICAL, 0 );
	sz_currentline->Add( new wxTextCtrl( this, ID_TX_STATUS_INFO, _( "" ), wxDefaultPosition, wxDefaultSize, wxTE_READONLY ), 1, wxALIGN_CENTER_VERTICAL, 0 );
	sz_monitor->Add( sz_currentline, 0, wxEXPAND | wxLEFT | wxRIGHT | wxBOTTOM, sizer_border );

	// Link quality
	sz_currentline = new wxBoxSizer( wxHORIZONTAL );
	sz_currentline->Add( new wxStaticText( this, ID_ST_LINKQUALITY, _( "Link Quality : " ), wxDefaultPosition, page1_labelsize ), 0, wxALIGN_CENTER_VERTICAL, 0 );
	sz_currentline->Add( new wxGauge( this, ID_GG_LINKQUALITY, 100, wxDefaultPosition, wxSize( -1, 12 ), wxGA_HORIZONTAL ), 1, wxALIGN_CENTER_VERTICAL, 0 );
	sz_monitor->Add( sz_currentline, 0, wxEXPAND | wxLEFT | wxRIGHT | wxBOTTOM, sizer_border );

	// Signal strength
	sz_currentline = new wxBoxSizer( wxHORIZONTAL );
	sz_currentline->Add( new wxStaticText( this, ID_ST_SIGSTRENGTH, _( "Signal Strength : " ), wxDefaultPosition, page1_labelsize ), 0, wxALIGN_CENTER_VERTICAL, 0 );
	sz_currentline->Add( new wxGauge( this, ID_GG_SIGSTRENGTH, 100, wxDefaultPosition, wxSize( -1, 12 ), wxGA_HORIZONTAL, wxDefaultValidator ), 1, wxALIGN_CENTER_VERTICAL, 0 );
	sz_monitor->Add( sz_currentline, 0, wxEXPAND | wxLEFT | wxRIGHT | wxBOTTOM, sizer_border );

////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////
	wxBoxSizer *sz_page_monitor = new wxBoxSizer( wxVERTICAL );
	sz_page_monitor->Add( sz_settings, 7, wxEXPAND | wxALL, 5 );
	sz_page_monitor->Add( sz_monitor, 3, wxEXPAND | wxLEFT | wxRIGHT | wxBOTTOM, 5 );
	SetSizer( sz_page_monitor );
	SetAutoLayout( true );


////////////////////////////////////Monitor page choices  setup	

	mMonitorChoice = new CMonitorChoiceSP( this ); // only for channel for now.

	wxArrayString operatingmode_ret = mMonitorChoice->ProvideStrings( ID_CH_OPERATINGMODE );
	( ( CAutoChoice * )FindWindowById( ID_CH_OPERATINGMODE ) )->SetStrings( operatingmode_ret ); 
	
	wxArrayString radioonoff_ret = mMonitorChoice->ProvideStrings( ID_CH_RADIOONOFF );
	( ( CAutoChoice * )FindWindowById( ID_CH_RADIOONOFF ) )->SetStrings( radioonoff_ret ); 

	wxArrayString inroaming_ret = mMonitorChoice->ProvideStrings( ID_CH_INROAMING );
	( ( CAutoChoice * )FindWindowById( ID_CH_INROAMING ) )->SetStrings( inroaming_ret ); 

	wxArrayString transmitrate_ret = mMonitorChoice->ProvideStrings( ID_CH_TRANSMITRATE );
	( ( CAutoChoice * )FindWindowById( ID_CH_TRANSMITRATE ) )->SetStrings( transmitrate_ret );

	wxArrayString channel_ret = mMonitorChoice->ProvideStrings( ID_CH_CHANNEL );
	( ( CAutoChoice * )FindWindowById( ID_CH_CHANNEL ) )->SetStrings( channel_ret ); 

	( ( CHistoryCombo * ) FindWindowById( ID_CB_ESSID ) )->SetMaxLength( MAX_SSID_LENGTH );
}
//=====================================================================================

CMonitorPage::~CMonitorPage( )
{
}

bool CMonitorPage::OnUpdateGauge( bool querydevice )
{
	CWinterModel *model = ( CWinterModel * ) CAccessor::Access( AID_CURRENTMODEL );
	CVnetDeviceBase *device = ( CVnetDeviceBase * ) CAccessor::Access( AID_CURRENTDEVICE );

	CDeviceConfigurationData *data = ( CDeviceConfigurationData * ) model->GetChild( MC_DEVICECONFIGURATIONDATA );

	if( querydevice == true )
	{
		if( device->GetDeviceConfig( data ) == false )
		{
			return false;
		}
	}

	int lq = data->GetLinkQuality( );
	float half_delta = ( 100.0 - ( float ) lq ) / 2.0;

	// lq == 0 means that there is a strong possibility that the radio is off.
	if( lq != 0 )
	{
		// Create kinda optical effect with the gauge.
		lq += ( int ) ( 0.7 * half_delta );
		lq += ( int ) ( 0.1 * half_delta + ( 0.1 * half_delta * rand( ) ) / ( RAND_MAX + 1.0 ) );
	}

	wxGauge *gglink = ( wxGauge * ) FindWindow( ID_GG_LINKQUALITY );
	gglink->SetValue( lq );

	// Absolute value is required because, sometimes, a negative RSSI value is returned.
	int rssi = abs( data->GetRssi( ) );

	if( rssi <= 100 )
	{
		wxGauge *ggrssi = ( wxGauge * ) FindWindow( ID_GG_SIGSTRENGTH );
		ggrssi->SetValue( rssi );
	}

	return true;
}

bool CMonitorPage::OnUpdateStatus( bool querydevice )
{
	CWinterModel *model = ( CWinterModel * ) CAccessor::Access( AID_CURRENTMODEL );
	CVnetDeviceBase *device = ( CVnetDeviceBase * ) CAccessor::Access( AID_CURRENTDEVICE );

	CDeviceConfigurationData *data = ( CDeviceConfigurationData * ) model->GetChild( MC_DEVICECONFIGURATIONDATA );

	if( querydevice == true )
	{
		if( device->GetDeviceConfig( data ) == false )
		{
			return false;
		}
	}

	int stationstate = data->GetStationState( );
	DisplayMessages( stationstate );

	if( stationstate != mStationState )
	{
		OnStationStateChanged( mStationState, stationstate );
		mStationState = stationstate;
	}

	return true;
}

void CMonitorPage::OnBtChange( wxCommandEvent &event )
{
	mInChangeMode = true;
	return;
}

void CMonitorPage::OnBtSubmit( wxCommandEvent &event )
{
#if ( USE_BUSY_INFO == 1 )// Once again some more ifdef's!
	wxBusyInfo busy( _( "Configuring device..." ) );
#endif // USE_BUSY_INFO == 1
	DEACTIVATE_TIMER( );
#if ( USE_BUSY_INFO == 0 )
	::wxBeginBusyCursor( );
#endif  // USE_BUSY_INFO == 0
	SubmitMonitorPageData( );
	mInChangeMode = false;
#if ( DELAY_AFTER_SUBMIT == 1 )
	::wxSleep( 1 );
#endif // DELAY_AFTER_SUBMIT == 1

	UpdateWinterPage( );

#if ( USE_BUSY_INFO == 0 )
	::wxEndBusyCursor( );
#endif  // USE_BUSY_INFO == 0
	ACTIVATE_TIMER( );

	return;
}

void CMonitorPage::OnBtCancel( wxCommandEvent &event )
{
	mInChangeMode = false;
	UpdateWinterPage( );
	return;
}

void CMonitorPage::SubmitMonitorPageData( )
{
	TransferToModel( NULL );
	CWinterModel *model = ( CWinterModel * ) CAccessor::Access( AID_CURRENTMODEL );
	CVnetDeviceBase *device = ( CVnetDeviceBase * ) CAccessor::Access( AID_CURRENTDEVICE );

	CDeviceConfigurationData *data = ( CDeviceConfigurationData * ) model->GetChild( MC_DEVICECONFIGURATIONDATA );

	if( device == NULL )
	{
		::wxMessageBox( _( "No device." ), wxT( "Winter" ), wxICON_HAND, ( CWinterFrame * ) CAccessor::Access( AID_FRAME ) );
		return;
	}
	
	if( device->SetDeviceConfig( data ) == true )
	{
		mIsDirty = true;
	}

	return;
}

void CMonitorPage::OnIdle( wxIdleEvent &event )
{
	bool enable_channel = mInChangeMode && !( ( ( CAutoChoice * ) FindWindow( ID_CH_OPERATINGMODE ) )->GetSelection( ) == 1 );
	bool enable_submit = mInChangeMode;

// Timer
	if( mInChangeMode == true )
	{
		DEACTIVATE_TIMER( );
	}

	if( mInChangeMode == false )
	{
		ACTIVATE_TIMER( );
	}

// Channel
	if( enable_channel != FindWindowById( ID_ST_CHANNEL )->IsEnabled( ) )
	{
		FindWindowById( ID_ST_CHANNEL )->Enable( enable_channel );
		FindWindowById( ID_CH_CHANNEL )->Enable( enable_channel );
	}

// Buttons
	if( FindWindowById( ID_BT_MONITORPAGE_CHANGE )->IsEnabled( ) == enable_submit )
	{
		FindWindowById( ID_BT_MONITORPAGE_CHANGE )->Enable( !enable_submit );
	}

	if( FindWindowById( ID_BT_MONITORPAGE_SUBMIT )->IsEnabled( ) != enable_submit )
	{
		FindWindowById( ID_BT_MONITORPAGE_SUBMIT )->Enable( enable_submit );
	}

	if( FindWindowById( ID_BT_MONITORPAGE_CANCEL )->IsEnabled( ) != enable_submit )
	{
		FindWindowById( ID_BT_MONITORPAGE_CANCEL )->Enable( enable_submit );
	}

// All other controls
	if( mInChangeMode != FindWindowById( ID_ST_OPERATINGMODE )->IsEnabled() )
	{
		FindWindowById( ID_ST_OPERATINGMODE )->Enable( mInChangeMode );
		FindWindowById( ID_CH_OPERATINGMODE )->Enable( mInChangeMode );

		CWinterStringsProvider *sp = ( CWinterStringsProvider * ) CAccessor::Access( AID_STRINGSPROVIDER );

		if( ( ( CAutoChoice * ) FindWindowById(  ID_CH_OPERATINGMODE  ) )->GetStringSelection( ) == sp->OperModeStrings[ 1 ] ) 
		{
			FindWindowById( ID_ST_CHANNEL )->Enable( false );
			FindWindowById( ID_CH_CHANNEL )->Enable( false );
		}
		else
		{
			FindWindowById( ID_ST_CHANNEL )->Enable( mInChangeMode );
			FindWindowById( ID_CH_CHANNEL )->Enable( mInChangeMode );
		}
		FindWindowById( ID_ST_ESSID )->Enable( mInChangeMode );
		FindWindowById( ID_CB_ESSID )->Enable( mInChangeMode );
		FindWindowById( ID_ST_RADIOONOFF )->Enable( mInChangeMode );
		FindWindowById( ID_CH_RADIOONOFF )->Enable( mInChangeMode );
		FindWindowById( ID_ST_MACADDRESS )->Enable( mInChangeMode );
		FindWindowById( ID_TX_MACADDRESS )->Enable( mInChangeMode );
		FindWindowById( ID_ST_INROAMING )->Enable( mInChangeMode );
		FindWindowById( ID_CH_INROAMING )->Enable( mInChangeMode );
		FindWindowById( ID_ST_TRANSMITRATE )->Enable( mInChangeMode );
		FindWindowById( ID_CH_TRANSMITRATE )->Enable( mInChangeMode );
	}

	event.Skip();
	return;
}

void CMonitorPage::OnChIntRoamingChanged( wxCommandEvent &event )
{
	update_channel_list( );
	return;
}

void CMonitorPage::update_channel_list( )
{
	CAutoChoice *chch = ( CAutoChoice * ) FindWindowById( ID_CH_CHANNEL );
	CAutoChoice *chin = ( CAutoChoice * ) FindWindowById( ID_CH_INROAMING );
	bool all_channels = true;

	if( chin->GetStringSelection( ) == _( "Enabled" ) )
	{
		all_channels = false;
	}

	wxString sel = chch->GetStringSelection( );

	chch->Clear( );

	CWinterModel *model = ( CWinterModel * ) CAccessor::Access( AID_CURRENTMODEL );
	CDeviceConfigurationData *data = ( CDeviceConfigurationData * ) model->GetChild( MC_DEVICECONFIGURATIONDATA );
	UCHAR *vector = data->GetChannelVector( );

	for( int c = 1; c <= 14; c++ )
	{
		if( all_channels == false && vector[ c ] == 0  )
		{
			continue;
		}

		chch->Append( wxString::Format( wxT( "%d" ), c ) );
	}

	if( chch->FindString( sel ) != -1 )
	{
		chch->SetStringSelection( sel );
	}

	return;
}

// Higher update level
void CMonitorPage::UpdateWinterPage( )
{
	CWinterModel *model = ( CWinterModel * ) CAccessor::Access( AID_CURRENTMODEL );
	CVnetDeviceBase *device = ( CVnetDeviceBase * ) CAccessor::Access( AID_CURRENTDEVICE );

	CDeviceConfigurationData *data = ( CDeviceConfigurationData * ) model->GetChild( MC_DEVICECONFIGURATIONDATA );
	if( device->GetDeviceConfig( data ) == false )
	{
		::wxMessageBox( _( "Failed to read configuration." ), wxT( "Winter" ), wxICON_HAND, ( CWinterFrame * ) CAccessor::Access( AID_FRAME ) );
		return;
	}

	CMacAddressData *mac_data = ( CMacAddressData * ) model->GetChild( MC_MACADDRESSDATA );
	if( device->GetMacAddress( mac_data ) == false )
	{
		::wxMessageBox( _( "Failed to read MAC address." ), wxT( "Winter" ), wxICON_HAND, ( CWinterFrame * ) CAccessor::Access( AID_FRAME ) );
		return;
	}

	TransferToControls( );
	return;
}


bool CMonitorPage::OnPageChanging( )
{
	mInChangeMode = false;
	return true;
}


int CMonitorPage::TestControlValues( )
{
	return 0;
}


int CMonitorPage::TransferToControls( CWinterModel *model )
{
	if( model == NULL )
	{
		model = ( CWinterModel * ) CAccessor::Access( AID_CURRENTMODEL );
	}

	CDeviceConfigurationData *device_data = ( CDeviceConfigurationData * ) model->GetChild( MC_DEVICECONFIGURATIONDATA );
	CMacAddressData *mac_data = ( CMacAddressData * ) model->GetChild( MC_MACADDRESSDATA );

	CWinterStringsProvider *sp = ( CWinterStringsProvider * ) CAccessor::Access( AID_STRINGSPROVIDER );

//int.roaming
	( ( CAutoChoice * ) FindWindow( ID_CH_INROAMING ) )->SetStringSelection( sp->RoamingStrings[ ( int ) device_data->GetInternationalRoaming( ) ] );
	update_channel_list( );

//channel
	( ( CAutoChoice * ) FindWindow( ID_CH_CHANNEL ) )->SetStringSelection( wxString::Format( _( "%d" ), ( int ) device_data->GetChannel( ) ) );

//radio
	( ( CAutoChoice * ) FindWindow( ID_CH_RADIOONOFF ) )->SetStringSelection( sp->RadioStrings[ ( int ) device_data->GetRadioIsOn( ) ] );

//oper mode
	( ( CAutoChoice * ) FindWindow( ID_CH_OPERATINGMODE ) )->SetSelection( ( int ) device_data->GetOperatingMode( ) );

//transmit rate
	( ( CAutoChoice * ) FindWindow( ID_CH_TRANSMITRATE ) )->SetSelection( ( int ) device_data->GetTxRate( ) );

// SSID
	CHistoryCombo *text = ( ( CHistoryCombo * ) FindWindow( ID_CB_ESSID ) );
	UCHAR *ssid = new UCHAR[ 33 ];
	memcpy( ssid, device_data->GetSSID( ), device_data->GetSSIDLength( ) );
	ssid[ device_data->GetSSIDLength( ) ] = 0;
        
	text->SetValueEx( wxString::Format( wxT( "%s" ), ssid ) );
	delete ssid;

// Mac address
	wxTextCtrl *mac_text = ( wxTextCtrl * ) FindWindow( ID_TX_MACADDRESS );
	mac_text->Clear( );
	UCHAR *mac_address = mac_data->GetMacAddress( );

	for( int c = 0; c < MAC_LENGTH - 1; c++ )
	{
		mac_text->AppendText( wxString::Format( wxT( "%02X:" ), mac_address[ c ] ) );
	}
	mac_text->AppendText( wxString::Format( wxT( "%02X" ), mac_address[ MAC_LENGTH - 1 ] ) );

//Get Station State
	DisplayMessages( device_data->GetStationState( ) );

//LinkQuality
	( ( wxGauge * ) FindWindow( ID_GG_LINKQUALITY ) )->SetValue( ( int ) device_data->GetLinkQuality( ) ); 

	return 0;
}


int CMonitorPage::TransferToModel( CWinterModel *model )
{
	if( model == NULL )
	{
		model = ( CWinterModel * ) CAccessor::Access( AID_CURRENTMODEL );
	}

	CDeviceConfigurationData *device_data = ( CDeviceConfigurationData * ) model->GetChild( MC_DEVICECONFIGURATIONDATA );

	CWinterStringsProvider *sp = ( CWinterStringsProvider * ) CAccessor::Access( AID_STRINGSPROVIDER );

//ssid
	UCHAR *ssid = new UCHAR[ MAX_SSID_LENGTH + 1 ];
	memset( ssid, 0, MAX_SSID_LENGTH + 1 );
	wxString tx_ssid = ( ( CHistoryCombo * ) FindWindow( ID_CB_ESSID ) )->GetValue( );
	strcpy( ( char * ) ssid, tx_ssid.c_str( ) );
	device_data->SetSSID( ssid );
	device_data->SetSSIDLength( tx_ssid.Len( ) );
	delete ssid;

//channel
	int channel = atoi( ( ( CAutoChoice * ) FindWindow( ID_CH_CHANNEL ) )->GetStringSelection( ).c_str( ) );
	device_data->SetChannel( channel );

//radio
	device_data->SetRadioIsOn( sp->RadioStrings.Index( ( ( CAutoChoice * ) FindWindow( ID_CH_RADIOONOFF ) )->GetStringSelection( ) ) );

//int.roaming
	device_data->SetInternationalRoaming( sp->RoamingStrings.Index( ( ( CAutoChoice * ) FindWindow( ID_CH_INROAMING ) )->GetStringSelection(  ) ) );

//oper mode
	device_data->SetOperatingMode( sp->OperModeStrings.Index( ( ( CAutoChoice * ) FindWindow( ID_CH_OPERATINGMODE ) )->GetStringSelection( ) ) );

//Transmit rate
	device_data->SetTxRate( sp->TransmitRateStrings.Index( ( ( CAutoChoice * ) FindWindow( ID_CH_TRANSMITRATE ) )->GetStringSelection( ) ) );

	return 0;
}

void CMonitorPage::DisplayErrorMessages( UCHAR stationstate )
{
	wxTextCtrl *info = ( wxTextCtrl * ) FindWindowById( ID_TX_STATUS_INFO );

	switch( stationstate )
	{
		case 1: 	//C80211_MGMT_SC_Unspecified:
			info->SetValue( _( "Unspecified error in Management Frame..." ) );
			break;
		case 10:	//C80211_MGMT_SC_SupportCapabilities:
			info->SetValue( _( "Support Capabilities error in Association Request..." ) );
			break;
		case 11:	//C80211_MGMT_SC_ReassDenied:
			info->SetValue( _( "Re-Association Denied" ) );
			break;
		case 12:	//C80211_MGMT_SC_AssDenied:
			info->SetValue( _( "Association Denied" ) );
			break;
		case 13:	//C80211_MGMT_SC_AuthAlgNotSupported:
			info->SetValue( _( "Authentication Algorithm not supported..." ) );
			break;
		case 14:	//C80211_MGMT_SC_AuthTransSeqNumError:
			info->SetValue( _( "Bad Authentication transaction sequence number..." ) );
			break;
		case 15:	//C80211_MGMT_SC_AuthRejectChallenge:
			info->SetValue( _( "Authentication challenge text rejection..." ) );
			break;
		case 16:	//C80211_MGMT_SC_AuthRejectTimeout:
			info->SetValue( _( "Authentication Timeout..." ) );
			break;
		case 17:	//C80211_MGMT_SC_AssDeniedHandleAP:
			info->SetValue( _( "Access Point can not handle any more stations..." ) );
			break;
		case 18:	//C80211_MGMT_SC_AssDeniedBSSRate:
			( ( wxTextCtrl * )FindWindowById( ID_TX_STATUS_INFO ) )->SetValue( _( "Association Denied (Supported Rates mismatch)..." ) );
			break;
		default:
			info->SetValue( _( "Failed to co-operate with the selected Access Point" ) );
	}

}

void CMonitorPage::OnStationStateChanged( int old_state, int new_state )
{
	TransferToControls( );
	return;
}

void CMonitorPage::DisplayMessages( UCHAR stationstate )
{
	CWinterModel *model = ( CWinterModel * ) CAccessor::Access( AID_CURRENTMODEL );
	CDeviceConfigurationData *device_data = ( CDeviceConfigurationData * ) model->GetChild( MC_DEVICECONFIGURATIONDATA );

	wxTextCtrl *info = ( wxTextCtrl * )FindWindowById( ID_TX_STATUS_INFO );

	CWinterStringsProvider *sp = ( CWinterStringsProvider * ) CAccessor::Access( AID_STRINGSPROVIDER );

	switch( stationstate )
	{
		case 0x00:
			info->SetValue( _( "Initializing" ) );
			break;
		case 0x01:
			info->SetValue( _( "Scanning" ) );
			break;
		case 0x02:
			info->SetValue( _( "Joining" ) );
			break;
		case 0x03:
			info->SetValue( _( "Authenticating" ) );
			break;
		case 0x04:
			info->SetValue( _( "Associating" ) );
			break;
		case 0x05:
			if( ( ( CAutoChoice * ) FindWindowById(  ID_CH_OPERATINGMODE  ) )->GetStringSelection( ) == sp->OperModeStrings[ 1 ] ) 
			{
				info->SetValue( _( "Associated" ) );
			}
			else
			{
				info->SetValue( _( "Ready" ) );
			}
                        info->AppendText( _( " - BSSID: " ) );
				
                        // TODO : Replace  this 5 with BSSID_LENGTH - 1
                        for( int c = 0; c < 5; c++ )
                        {
                                info->AppendText( wxString::Format( wxT( "%02X:" ), device_data->GetBSSID( )[ c ] ) );
                        }
                        info->AppendText( wxString::Format( wxT( "%02X" ), device_data->GetBSSID( )[ 5 ] ) );
			break;
		case 0x06:
			info->SetValue( _( "Station Out of Range" ) );
			break;
		case 0x07:
			info->SetValue( _( "Disassociating" ) );
			break;
		case 0x08:
			info->SetValue( _( "Disauthenticating" ) );
			break;
		case 0x09:
			info->SetValue( _( "Reassociating" ) );
			break;
		case 0x0a:
			info->SetValue( _( "Forced Joining" ) );
			break;
		case 0x0b:
			info->SetValue( _( "ReConfiguring" ) );
			break;
		case 0x0c:
			info->SetValue( _( "FW_Upgrade" ) );
			break;
		case 0x0d:
			info->SetValue( _( "Force Joining Failure" ) );
			break;
		case 0x0e:
			info->SetValue( _( "Resetting" ) );
			break;
		case 0x0f:
			info->SetValue( _( "Reset Pending" ) );
			break;
		case 0x13:
			info->SetValue( _( "Radio OFF" ) );
			break;
		case 0xf0:
			info->SetValue( _( "No card Present" ) );
			break;
		case 0xf1:
			if( device_data->GetMgmtErrorCode( ) == 0xa0 )
			{
				info->SetValue( _( "Enable Encryption to Join this BSS" ) );
				break;
			}
			else if( device_data->GetMgmtErrorCode( ) == 0xa1 )
			{
				info->SetValue( _ ( "Disable Encryption to Join this BSS" ) );
				break;
			}
			else
			{
				DisplayErrorMessages( ( UCHAR )device_data->GetStationState ( ) );
			}
			break;

		case 0xfd:
			info->SetValue( _( "Failed to Find SSID. Check your Settings." ) );
			break;
		default:
			info->SetValue( wxString::Format( _( "Unknown station state code : 0x%02X" ), stationstate ) );
	}

	return;
}


