/* Profile.h */
/*******************************************************************************************
	Copyright 2002-2003 ATMEL Corporation.

	This file is part of ATMEL Wireless LAN Drivers.

	ATMEL Wireless LAN Drivers is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	ATMEL Wireless LAN Drivers is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with ATMEL Wireless LAN Drivers; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*******************************************************************************************/

#ifndef __PROFILE_H__
#define __PROFILE_H__

#include <wx/wx.h>
#include <wx/dynarray.h>
#include "WinterModel.h"
#include "xvnet.h"


class CDeviceConfigurationData;
class CWepInfoData;


class CWinterProfile
{
protected:
	wxString mName;
	DEVICE_CONFIGURATION *mDevData;
	WEP_INFO *mWepData;

public:
	CWinterProfile( const wxString &name = _( "" ) );
	virtual ~CWinterProfile( );

public:
	void SetName( const wxString &name );
	wxString GetName( );

public:
	void Import( CWinterModel *model = NULL );
	void Import( CDeviceConfigurationData *data );
	void Import( CWepInfoData *data );

	void Export( CWinterModel *model = NULL );
	void Export( CDeviceConfigurationData *data );
	void Export( CWepInfoData *data );
};


WX_DEFINE_ARRAY( CWinterProfile *, ArrayProfiles );

class CWinterProfileManager
{
protected:
	ArrayProfiles mProfiles;

public:
	CWinterProfileManager( );
	virtual ~CWinterProfileManager( );

public:
	bool AddProfile( CWinterProfile *profile );
	bool RemoveProfile( CWinterProfile *profile );

public:
	int CountProfiles( );
	CWinterProfile *GetProfile( int index );
	CWinterProfile *GetProfile( const wxString &name );

	CWinterProfile *operator[]( int index );

public:
	bool LoadFile( const wxString &str_filename = _( "" ) );
	bool SaveFile( const wxString &str_filename = _( "" ) ); 
};


class CWinterProfileManagerSingleton
{
protected:
	static CWinterProfileManager *mManager;

protected:
	CWinterProfileManagerSingleton( );

public:
	virtual ~CWinterProfileManagerSingleton( );

public:
	static CWinterProfileManager *Get( );
};


class CWepKeyFilter
{
protected:
	wxString mHex;
	wxString mAscii;

public:
	CWepKeyFilter( );
	CWepKeyFilter( const wxString &input, bool ishex = true );
	virtual ~CWepKeyFilter( );

public:
	void Clear( );

	bool SetWepKey( const wxString &key, bool ishex = true );
	wxString GetWepKey( bool ishex = true );
};


#endif // __PROFILE_H__


