/* ProfilesPage.cpp */
/*******************************************************************************************
	Copyright 2002-2003 ATMEL Corporation.

	This file is part of ATMEL Wireless LAN Drivers.

	ATMEL Wireless LAN Drivers is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	ATMEL Wireless LAN Drivers is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with ATMEL Wireless LAN Drivers; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*******************************************************************************************/

#include "ProfilesPage.h"
#include "Profile.h"
#include "Accessor.h"
#include "WinterModel.h"
#include "WinterData.h"
#include "Device.h"
#include "WinterLang.h"
#include "WinterConfig.h"
#include "WinterTimer.h"
#include "HRStrings.h"
#include "Decls.h"

#include "ChoiceCtrl.h"
#include "MonitorPage.h"
#include "EncryptionPage.h"
#include "WinterSetup.h"

#include <wx/wx.h>
#include <wx/statline.h>
#include <wx/notebook.h>
#include <wx/busyinfo.h>


BEGIN_EVENT_TABLE( CProfilesPage, CWinterPage )
	EVT_IDLE( CProfilesPage::OnIdle )
	EVT_LISTBOX( ID_CL_PROFILESPAGE_PROFILES, CProfilesPage::OnLbSelected )
	EVT_LISTBOX_DCLICK( ID_CL_PROFILESPAGE_PROFILES, CProfilesPage::OnLbDClicked )
	EVT_CHECKLISTBOX( ID_CL_PROFILESPAGE_PROFILES, CProfilesPage::OnClProfilesToggled )
	EVT_BUTTON( ID_BT_PROFILESPAGE_ADD, CProfilesPage::OnBtAdd )
	EVT_BUTTON( ID_BT_PROFILESPAGE_REMOVE, CProfilesPage::OnBtRemove )
	EVT_BUTTON( ID_BT_PROFILESPAGE_LOAD, CProfilesPage::OnBtLoad )
	EVT_BUTTON( ID_BT_PROFILESPAGE_SAVE, CProfilesPage::OnBtSave )
	EVT_CHOICE( ID_CH_PROFILESPAGE_LANG, CProfilesPage::OnChLang )
END_EVENT_TABLE( )


// CProfilesPage class implementation
//====================================
CProfilesPage::CProfilesPage( wxWindow *parent, int id, const wxPoint &pos, const wxSize &size, long style ) : CWinterPage( parent, id )
{
	wxBoxSizer *sz_bt1 = new wxBoxSizer( wxHORIZONTAL );
	sz_bt1->Add( new wxButton( this, ID_BT_PROFILESPAGE_LOAD, _( "&Load" ) ), 1, wxEXPAND | wxALL, 5 );
	sz_bt1->Add( new wxButton( this, ID_BT_PROFILESPAGE_SAVE, _( "&Save" ) ), 1, wxEXPAND | wxTOP | wxBOTTOM | wxRIGHT, 5 );

	wxBoxSizer *sz_bt2 = new wxBoxSizer( wxHORIZONTAL );
	sz_bt2->Add( new wxButton( this, ID_BT_PROFILESPAGE_ADD, _( "&Add" ) ), 1, wxEXPAND | wxALL, 5 );
	sz_bt2->Add( new wxButton( this, ID_BT_PROFILESPAGE_REMOVE, _( "&Remove" ) ), 1, wxEXPAND | wxTOP | wxBOTTOM | wxRIGHT, 5 );

	wxBoxSizer *sz_bt = new wxBoxSizer( wxVERTICAL );
	sz_bt->Add( sz_bt1, 0, wxEXPAND, 0 );
	sz_bt->Add( new wxStaticLine( this, -1 ), 0, wxEXPAND | wxALL, 3 );
	sz_bt->Add( sz_bt2, 0, wxEXPAND, 0 );

	wxBoxSizer *sz_lbbt = new wxBoxSizer( wxVERTICAL );
	sz_lbbt->Add( new CRadioListBox( this, ID_CL_PROFILESPAGE_PROFILES ), 1, wxEXPAND | wxALL, 5 ); 
	sz_lbbt->Add( sz_bt, 0, wxEXPAND, 0 );

	wxBoxSizer *sz_profiles = new wxStaticBoxSizer( new wxStaticBox( this, -1, _( "Device profile" ) ), wxHORIZONTAL );
	sz_profiles->Add( sz_lbbt, 1, wxEXPAND, 0 );
	sz_profiles->Add( new CPreviewPanel( this, ID_PN_PROFILESPAGE_PREVIEW ), 2, wxEXPAND | wxBOTTOM | wxLEFT | wxRIGHT, 5 );

	wxBoxSizer *sz_langline = new wxBoxSizer( wxHORIZONTAL );
	sz_langline->Add( new wxStaticText( this, ID_ST_PROFILESPAGE_LANG, _( "Language : " ) ), 0, wxALL | wxALIGN_CENTER_VERTICAL, 3 );
	sz_langline->Add( new CAutoChoice( this, ID_CH_PROFILESPAGE_LANG, wxDefaultPosition, wxDefaultSize, CWinterSupportedLanguages::GetAllLanguages( ) ), 1, wxALL | wxALIGN_CENTER_VERTICAL, 6 );

	wxBoxSizer *sz_options = new wxStaticBoxSizer( new wxStaticBox( this, -1, _( "Startup Options" ) ), wxVERTICAL );
	sz_options->Add( new wxStaticText( this, ID_ST_PROFILESPAGE_STARTUPOPTIONS, _( "Note that you will have to restart the application in order the language changes to take effect." ) ), 0, wxALL, 3 );
	sz_options->Add( sz_langline, 1, wxEXPAND, 0 );

	wxBoxSizer *sz_main = new wxBoxSizer( wxVERTICAL );
	sz_main->Add( sz_profiles, 3, wxEXPAND | wxALL, 5 );
	sz_main->Add( sz_options, 1, wxEXPAND | wxLEFT | wxRIGHT | wxBOTTOM, 5 );

	SetSizer( sz_main );
	SetAutoLayout( true );

	Layout( );
}

CProfilesPage::~CProfilesPage( )
{
}

void CProfilesPage::UpdateWinterPage( )
{
	CWinterProfileManager *pmanager = ( CWinterProfileManager * ) CAccessor::Access( AID_PROFILEMANAGER );
	CWinterConfig *config = ( CWinterConfig * ) CAccessor::Access( AID_CONFIG );

	wxCheckListBox *clprofiles = ( wxCheckListBox * ) FindWindowById( ID_CL_PROFILESPAGE_PROFILES );
	CAutoChoice *chlang = ( CAutoChoice * ) FindWindowById( ID_CH_PROFILESPAGE_LANG );

	wxString lang = config->GetLangName( );
	wxString startup_profile = config->GetStartupProfile( );
	wxString tmp;

	clprofiles->Clear( );

	int count = pmanager->CountProfiles( );

	for( int c = 0; c < count; c++ )
	{
		tmp = pmanager->GetProfile( c )->GetName( );
		clprofiles->Append( tmp );

		if( tmp == startup_profile )
		{
			clprofiles->Check( c, true );
		}
	}

	if( chlang->FindString( lang ) == -1 )
	{
		chlang->SetSelection( 0 );
	}
	else
	{
		chlang->SetStringSelection( lang );
	}

	return;
}

void CProfilesPage::ActivateProfile( CWinterProfile *profile )
{
	CWinterModel *model = new CWinterModel( );
	profile->Export( model );

	CMonitorPage *pgmonitor = ( CMonitorPage * ) FindWindowById( ID_PN_MONITORPAGE );
	CEncryptionPage *pgencryption = ( CEncryptionPage * ) FindWindowById( ID_PN_ENCRYPTIONPAGE );
	CVnetDeviceBase *device = ( CVnetDeviceBase * ) CAccessor::Access( AID_CURRENTDEVICE );
	CDeviceConfigurationData *dev_data = ( CDeviceConfigurationData * ) model->GetChild( MC_DEVICECONFIGURATIONDATA );
	CWepInfoData *wep_data = ( CWepInfoData * ) model->GetChild( MC_WEPINFODATA );

	DEACTIVATE_TIMER( );
	device->SetDeviceConfig( dev_data );
#if ( DELAY_AFTER_SUBMIT == 1 )
	::wxSleep( 1 );
#endif // DELAY_AFTER_SUBMIT == 1
	device->SetWepInfo( wep_data );
#if ( DELAY_AFTER_SUBMIT == 1 )
	::wxSleep( 1 );
#endif // DELAY_AFTER_SUBMIT == 1
	pgmonitor->UpdateWinterPage( );
	pgencryption->UpdateWinterPage( );
	ACTIVATE_TIMER( );

	delete model;
	return;
}

bool CProfilesPage::OnPageChanging( )
{
	CPreviewPanel *pnpreview = ( CPreviewPanel * ) FindWindowById( ID_PN_PROFILESPAGE_PREVIEW );
	pnpreview->ClearPreview( );

	return true;
}

void CProfilesPage::OnIdle( wxIdleEvent &event )
{
	int sel = ( ( wxListBox * ) FindWindowById( ID_CL_PROFILESPAGE_PROFILES ) )->GetSelection( );

	bool any_selected = ( sel != -1 );

	FindWindowById( ID_BT_PROFILESPAGE_REMOVE )->Enable( any_selected );
	FindWindowById( ID_BT_PROFILESPAGE_LOAD )->Enable( any_selected );
	FindWindowById( ID_BT_PROFILESPAGE_SAVE )->Enable( any_selected );

	event.Skip( );
	return;
}

void CProfilesPage::OnLbDClicked( wxCommandEvent &event )
{
	CWinterProfileManager *pmanager = ( CWinterProfileManager * ) CAccessor::Access( AID_PROFILEMANAGER );
	CWinterProfile *profile = pmanager->GetProfile( event.GetSelection( ) );

	ActivateProfile( profile );

	if( OnPageChanging( ) == true )
	{
		( ( wxNotebook * ) FindWindowById( ID_NB_MAINNOTEBOOK ) )->SetSelection( 0 );
	}

	event.Skip( );
	return;
}

void CProfilesPage::OnLbSelected( wxCommandEvent &event )
{
	CWinterProfileManager *pmanager = ( CWinterProfileManager * ) CAccessor::Access( AID_PROFILEMANAGER );
	CWinterProfile *profile = pmanager->GetProfile( event.GetSelection( ) );
	CPreviewPanel *pnpreview = ( CPreviewPanel * ) FindWindowById( ID_PN_PROFILESPAGE_PREVIEW );

	pnpreview->DoPreview( profile );
	return;
}

void CProfilesPage::OnBtAdd( wxCommandEvent &event )
{
	CProfileDialog dlg( ( wxFrame * ) CAccessor::Access( AID_FRAME ) );
	dlg.CentreOnParent( );
	if( dlg.ShowModal( ) == wxID_OK )
	{
		UpdateWinterPage( );
	}

	return;
}

void CProfilesPage::OnBtRemove( wxCommandEvent &event )
{
	CWinterProfileManager *pmanager = ( CWinterProfileManager * ) CAccessor::Access( AID_PROFILEMANAGER );
	CWinterConfig *config = ( CWinterConfig * ) CAccessor::Access( AID_CONFIG );
	CRadioListBox *clprofiles = ( CRadioListBox * ) FindWindowById( ID_CL_PROFILESPAGE_PROFILES );

	CWinterProfile *current_profile;
	int selection = clprofiles->GetSelection( );

	if ( selection != -1 ) 
	{
		current_profile = pmanager->GetProfile( selection );

		if( current_profile != NULL )
		{
			pmanager->RemoveProfile( current_profile );

			if( clprofiles->GetChecked( ) == selection )
			{
				config->SetStartupProfile( wxT( "" ) );
			}

			clprofiles->Delete( selection );

			( ( CAutoChoice *) FindWindowById( ID_CH_MAINPANEL_PROFILES ) )->Delete( selection );
			OnPageChanging( );		
		}
	}
	
	return;
}

void CProfilesPage::OnBtLoad( wxCommandEvent &event )
{
#if ( USE_BUSY_CURSOR == 1 )
	wxBusyInfo info( _( "Applying profile settings..." ) );
#endif // USE_BUSY_CURSOR == 1
	wxListBox *lbprofiles = ( wxListBox * ) FindWindowById( ID_CL_PROFILESPAGE_PROFILES );
	CWinterProfileManager *pmanager = ( CWinterProfileManager * ) CAccessor::Access( AID_PROFILEMANAGER );
	CWinterProfile *profile = pmanager->GetProfile( lbprofiles->GetSelection( ) );

	ActivateProfile( profile );

	if( OnPageChanging( ) == true )
	{
		( ( wxNotebook * ) FindWindowById( ID_NB_MAINNOTEBOOK ) )->SetSelection( 0 );
	}

	return;
}

void CProfilesPage::OnBtSave( wxCommandEvent &event )
{
	CWinterProfileManager *pmanager = ( CWinterProfileManager * ) CAccessor::Access( AID_PROFILEMANAGER );
	wxListBox *lbprofiles = ( wxListBox * ) FindWindowById( ID_CL_PROFILESPAGE_PROFILES );

	wxString profilename = lbprofiles->GetStringSelection( );
	CWinterProfile *profile = pmanager->GetProfile( profilename );
	profile->Import( );
	UpdateWinterPage( );
	return;
}

void CProfilesPage::OnChLang( wxCommandEvent &event )
{
	CWinterConfig *config = ( CWinterConfig * ) CAccessor::Access( AID_CONFIG );
	CAutoChoice *chlang = ( CAutoChoice * ) FindWindowById( ID_CH_PROFILESPAGE_LANG );

	config->SetLangName( chlang->GetStringSelection( ) );
	return;
}

void CProfilesPage::OnClProfilesToggled( wxCommandEvent &event )
{
	CWinterConfig *config = ( CWinterConfig * ) CAccessor::Access( AID_CONFIG );
	CRadioListBox *clprofiles = ( CRadioListBox * ) FindWindowById( ID_CL_PROFILESPAGE_PROFILES );

	int sel = clprofiles->GetChecked( );
	wxString profile_name = wxT( "" );

	if( sel != -1 )
	{
		profile_name = clprofiles->GetString( sel );
	}

	config->SetStartupProfile( profile_name );
	event.Skip( );
	return;
}


// CProfileDialog class implementation
//=====================================
BEGIN_EVENT_TABLE( CProfileDialog, wxDialog )
	EVT_BUTTON( wxID_OK, CProfileDialog::OnBtAdd )
END_EVENT_TABLE()


CProfileDialog::CProfileDialog( wxWindow *parent, int id, const wxString &title, const wxPoint &pos, const wxSize &size, long style ) : wxDialog( parent, id, title, pos, size, style )
{
	wxBoxSizer *sz_txinput = new wxBoxSizer( wxHORIZONTAL );
	sz_txinput->Add( new wxStaticText( this , ID_ST_DIALOG,  _( "Profile name : " ), wxDefaultPosition, wxDefaultSize ), 0, wxALIGN_CENTER_VERTICAL | wxALL, 5 );
	sz_txinput->Add( new wxTextCtrl( this , ID_TX_DIALOG, wxT( "" ), wxDefaultPosition, wxDefaultSize ), 1, wxALIGN_CENTER_VERTICAL | wxTOP | wxBOTTOM | wxRIGHT, 5 );

	wxBoxSizer *sz_buttons = new wxBoxSizer( wxHORIZONTAL );
	sz_buttons->Add( 1, 1, 1, wxEXPAND, 0 );
	sz_buttons->Add( new wxButton( this, wxID_OK, _( "Add" ) ) , 0, wxALL, 5 );
	sz_buttons->Add( new wxButton( this, wxID_CANCEL, _( "Cancel" ) ), 0, wxTOP | wxBOTTOM | wxRIGHT, 5 );

	wxBoxSizer *sz_top = new wxBoxSizer( wxVERTICAL );
	sz_top->Add( sz_txinput, 1, wxEXPAND, 5 );
	sz_top->Add( sz_buttons, 0, wxEXPAND, 5 );

	SetSizer( sz_top );
	SetAutoLayout( true );
	Layout( );

	( ( wxButton * ) FindWindowById( wxID_OK ) )->SetDefault( );
	( ( wxTextCtrl * ) FindWindowById( ID_TX_DIALOG ) )->SetFocus( );

	SetSizeHints( 300, 80 );
}

CProfileDialog::~CProfileDialog( )
{

}

void CProfileDialog::OnBtAdd( wxCommandEvent &event )
{
	AddProfile( );
	event.Skip( );
	return;
}

bool CProfileDialog::AddProfile( )
{
	CWinterProfileManager *pmanager = ( CWinterProfileManager * ) CAccessor::Access( AID_PROFILEMANAGER );
	wxString profile_name = ( ( wxTextCtrl * ) FindWindowById( ID_TX_DIALOG ) )->GetValue( );

	if( profile_name.Trim().IsEmpty( ) == true )
	{
		::wxMessageBox( _( "Enter a profile name!" ), wxT( "Winter" ), wxICON_HAND );
		return false;
	}

	mProfileNew = new CWinterProfile( profile_name );
	if( pmanager->GetProfile( profile_name ) != NULL )
	{
		switch( ::wxMessageBox( _( "There is already a profile with this name.\nOverwrite?" ), wxT( "Winter" ), wxYES_NO | wxICON_QUESTION, this ) )
		{
			case wxYES:
				delete mProfileNew;
				mProfileNew = pmanager->GetProfile( profile_name );
				break;
			case wxNO:
				delete mProfileNew;
				return false;
			default:
				::wxMessageBox( _( "You have found a bug! Please report." ), _( "Congratulations!" ) );
				break;
		}
	}

	mProfileNew->Import( ); 
	pmanager->AddProfile( mProfileNew );

	( ( CAutoChoice *) FindWindowById( ID_CH_MAINPANEL_PROFILES ) )->Append( mProfileNew->GetName( ) );

	return true;
}


// CPreviewPanel class implementation
//====================================
CPreviewPanel::CPreviewPanel( wxWindow *parent, int id ) : wxPanel( parent, id )
{
	wxSize labelsize( 150, -1 );
	int sizer_border = 3;

	wxBoxSizer *sz_currentline;
	wxBoxSizer *sz_prv = new wxBoxSizer( wxVERTICAL );

	// Operating mode
	sz_currentline = new wxBoxSizer( wxHORIZONTAL );
	sz_currentline->Add( new wxStaticText( this, ID_ST_PROFILESPAGE_PRV_OPMODE, _( "Operating Mode :" ), wxDefaultPosition, labelsize ), 0, wxALIGN_CENTER_VERTICAL, 0 );
	sz_currentline->Add( new wxStaticText( this, ID_ST_PROFILESPAGE_PRV_OPMODE_VAL, wxT( "" ), wxDefaultPosition ), 1, wxALIGN_CENTER_VERTICAL, 0 );
	sz_prv->Add( sz_currentline, 0, wxEXPAND | wxALL, sizer_border );

	// Channel   
	sz_currentline = new wxBoxSizer( wxHORIZONTAL );
	sz_currentline->Add( new wxStaticText( this, ID_ST_PROFILESPAGE_PRV_CHANNEL, _( "Channel : " ), wxDefaultPosition, labelsize ), 0, wxALIGN_CENTER_VERTICAL, 0 );
	sz_currentline->Add( new wxStaticText( this, ID_ST_PROFILESPAGE_PRV_CHANNEL_VAL, wxT( "" ), wxDefaultPosition ), 1, wxALIGN_CENTER_VERTICAL, 0 );
	sz_prv->Add( sz_currentline, 0, wxEXPAND | wxLEFT | wxRIGHT | wxBOTTOM, sizer_border );

	// ESSID
	sz_currentline = new wxBoxSizer( wxHORIZONTAL );
	sz_currentline->Add( new wxStaticText( this, ID_ST_PROFILESPAGE_PRV_SSID, wxT( "ESSID : " ), wxDefaultPosition, labelsize ), 0, wxALIGN_CENTER_VERTICAL, 0 );
	sz_currentline->Add( new wxStaticText( this, ID_ST_PROFILESPAGE_PRV_SSID_VAL, wxT( "" ), wxDefaultPosition ), 1, wxALIGN_CENTER_VERTICAL, 0 );
	sz_prv->Add( sz_currentline, 0, wxEXPAND | wxLEFT | wxRIGHT | wxBOTTOM, sizer_border );

	// Radio on/off
	sz_currentline = new wxBoxSizer( wxHORIZONTAL );
	sz_currentline->Add( new wxStaticText( this, ID_ST_PROFILESPAGE_PRV_RADIO, _( "Radio On/Off :" ), wxDefaultPosition, labelsize ), 0, wxALIGN_CENTER_VERTICAL, 0 );
	sz_currentline->Add( new wxStaticText( this, ID_ST_PROFILESPAGE_PRV_RADIO_VAL, wxT( "" ), wxDefaultPosition ), 1, wxALIGN_CENTER_VERTICAL, 0 );
	sz_prv->Add( sz_currentline, 0, wxEXPAND | wxLEFT | wxRIGHT | wxBOTTOM, sizer_border );

	// Int Roaming
	sz_currentline = new wxBoxSizer( wxHORIZONTAL );
	sz_currentline->Add( new wxStaticText( this, ID_ST_PROFILESPAGE_PRV_INTROAMING, _( "Int. Roaming :" ), wxDefaultPosition, labelsize ), 0, wxALIGN_CENTER_VERTICAL, 0 );
	sz_currentline->Add( new wxStaticText( this, ID_ST_PROFILESPAGE_PRV_INTROAMING_VAL, wxT( "" ), wxDefaultPosition ), 1, wxALIGN_CENTER_VERTICAL, 0 );
	sz_prv->Add( sz_currentline, 0, wxEXPAND | wxLEFT | wxRIGHT | wxBOTTOM, sizer_border );

	// Transmit rate
	sz_currentline = new wxBoxSizer( wxHORIZONTAL );
	sz_currentline->Add( new wxStaticText( this, ID_ST_PROFILESPAGE_PRV_TRRATE, _( "Transmit Rate :" ), wxDefaultPosition, labelsize ), 0, wxALIGN_CENTER_VERTICAL, 0 );
	sz_currentline->Add( new wxStaticText( this, ID_ST_PROFILESPAGE_PRV_TRRATE_VAL, wxT( "" ), wxDefaultPosition ), 1, wxALIGN_CENTER_VERTICAL, 0 );
	sz_prv->Add( sz_currentline, 0, wxEXPAND | wxLEFT | wxRIGHT | wxBOTTOM, sizer_border );

	// Preamble Type
	sz_currentline = new wxBoxSizer( wxHORIZONTAL );
	sz_currentline->Add( new wxStaticText( this, ID_ST_PROFILESPAGE_PRV_PRTYPE, wxT( "Preamble :" ), wxDefaultPosition, labelsize ), 0, wxALIGN_CENTER_VERTICAL, 0 );
	sz_currentline->Add( new wxStaticText( this, ID_ST_PROFILESPAGE_PRV_PRTYPE_VAL, wxT( "" ), wxDefaultPosition ), 1, wxALIGN_CENTER_VERTICAL, 0 );
	sz_prv->Add( sz_currentline, 0, wxEXPAND | wxLEFT | wxRIGHT | wxBOTTOM, sizer_border );

	// Power Mode
	sz_currentline = new wxBoxSizer( wxHORIZONTAL );
	sz_currentline->Add( new wxStaticText( this, ID_ST_PROFILESPAGE_PRV_PWRMODE, _( "Power Mode :" ), wxDefaultPosition, labelsize ), 0, wxALIGN_CENTER_VERTICAL, 0 );
	sz_currentline->Add( new wxStaticText( this, ID_ST_PROFILESPAGE_PRV_PWRMODE_VAL, wxT( "" ), wxDefaultPosition ), 1, wxALIGN_CENTER_VERTICAL, 0 );
	sz_prv->Add( sz_currentline, 0, wxEXPAND | wxLEFT | wxRIGHT | wxBOTTOM, sizer_border );

	// Fragmentation threshold
	sz_currentline = new wxBoxSizer( wxHORIZONTAL );
	sz_currentline->Add( new wxStaticText( this, ID_ST_PROFILESPAGE_PRV_FRAGTHRSHLD, _( "Fragmentation threshold :" ), wxDefaultPosition, labelsize ), 0, wxALIGN_CENTER_VERTICAL, 0 );
	sz_currentline->Add( new wxStaticText( this, ID_ST_PROFILESPAGE_PRV_FRAGTHRSHLD_VAL, wxT( "" ), wxDefaultPosition ), 1, wxALIGN_CENTER_VERTICAL, 0 );
	sz_prv->Add( sz_currentline, 0, wxEXPAND | wxLEFT | wxRIGHT | wxBOTTOM, sizer_border );

	// RTS/CTS threshold
	sz_currentline = new wxBoxSizer( wxHORIZONTAL );
	sz_currentline->Add( new wxStaticText( this, ID_ST_PROFILESPAGE_PRV_RTSCTSTHR, _( "RTS/CTS threshold :" ), wxDefaultPosition, labelsize ), 0, wxALIGN_CENTER_VERTICAL, 0 );
	sz_currentline->Add( new wxStaticText( this, ID_ST_PROFILESPAGE_PRV_RTSCTSTHR_VAL, wxT( "" ), wxDefaultPosition ), 1, wxALIGN_CENTER_VERTICAL, 0 );
	sz_prv->Add( sz_currentline, 0, wxEXPAND | wxLEFT | wxRIGHT | wxBOTTOM, sizer_border );
	sz_prv->Add( new wxStaticLine( this, -1 ), 0, wxEXPAND, 3 );

	//Encryption
	sz_currentline = new wxBoxSizer( wxHORIZONTAL );
	sz_currentline->Add( new wxStaticText( this, ID_ST_PROFILESPAGE_PRV_ENCRYPTION, _( "Encryption :" ), wxDefaultPosition, labelsize ), 0, wxALIGN_CENTER_VERTICAL, 0 );
	sz_currentline->Add( new wxStaticText( this, ID_ST_PROFILESPAGE_PRV_ENCRYPTION_VAL, wxT( "" ), wxDefaultPosition ), 1, wxALIGN_CENTER_VERTICAL, 0 );
	sz_prv->Add( sz_currentline, 0, wxEXPAND | wxLEFT | wxRIGHT | wxBOTTOM, sizer_border );

	//WepKeyToUse
	sz_currentline = new wxBoxSizer( wxHORIZONTAL );
	sz_currentline->Add( new wxStaticText( this, ID_ST_PROFILESPAGE_PRV_WEPKEYTOUSE, wxT( "" ), wxDefaultPosition, labelsize ), 0, wxALIGN_CENTER_VERTICAL, 0 );
	sz_currentline->Add( new wxStaticText( this, ID_ST_PROFILESPAGE_PRV_WEPKEYTOUSE_VAL, wxT( "" ), wxDefaultPosition ), 1, wxALIGN_CENTER_VERTICAL, 0 );
	sz_prv->Add( sz_currentline, 0, wxEXPAND | wxLEFT | wxRIGHT | wxBOTTOM, sizer_border );

	//WepMode
	sz_currentline = new wxBoxSizer( wxHORIZONTAL );
	sz_currentline->Add( new wxStaticText( this, ID_ST_PROFILESPAGE_PRV_WEPMODE, wxT( "" ), wxDefaultPosition, labelsize ), 0, wxALIGN_CENTER_VERTICAL, 0 );
	sz_currentline->Add( new wxStaticText( this, ID_ST_PROFILESPAGE_PRV_WEPMODE_VAL, wxT( "" ), wxDefaultPosition ), 1, wxALIGN_CENTER_VERTICAL, 0 );
	sz_prv->Add( sz_currentline, 0, wxEXPAND | wxLEFT | wxRIGHT | wxBOTTOM, sizer_border );

	//Authentication Type
	sz_currentline = new wxBoxSizer( wxHORIZONTAL );
	sz_currentline->Add( new wxStaticText( this, ID_ST_PROFILESPAGE_PRV_AUTHENTICATIONTYPE, wxT( "" ), wxDefaultPosition, labelsize ), 0, wxALIGN_CENTER_VERTICAL, 0 );
	sz_currentline->Add( new wxStaticText( this, ID_ST_PROFILESPAGE_PRV_AUTHENTICATIONTYPE_VAL, wxT( "" ), wxDefaultPosition ), 1, wxALIGN_CENTER_VERTICAL, 0 );
	sz_prv->Add( sz_currentline, 0, wxEXPAND | wxLEFT | wxRIGHT | wxBOTTOM, sizer_border );

	// Wep keys not shown
	sz_currentline = new wxBoxSizer( wxHORIZONTAL );
	sz_currentline->Add( new wxStaticText( this, ID_ST_PROFILESPAGE_PRV_WEPNOTSHOWN, wxT( "" ), wxDefaultPosition, labelsize ), 1, wxALIGN_CENTER_VERTICAL | wxALIGN_CENTER_HORIZONTAL, 0 );
	sz_prv->Add( sz_currentline, 0, wxEXPAND | wxLEFT | wxRIGHT | wxBOTTOM, sizer_border );

	SetSizer( sz_prv );
	SetAutoLayout( true );
	Layout( );
}

CPreviewPanel::~CPreviewPanel( )
{
}

void CPreviewPanel::DoPreview( CWinterProfile *profile )
{
	if( profile == NULL )
	{
		ClearPreview( );
		return;
	}

	CWinterModel *model = new CWinterModel( );
	profile->Export( model );

	CDeviceConfigurationData *dev_data = ( CDeviceConfigurationData * ) model->GetChild( MC_DEVICECONFIGURATIONDATA );
	CWepInfoData *wep_data = ( CWepInfoData * ) model->GetChild( MC_WEPINFODATA );
	CWinterStringsProvider *sp = ( CWinterStringsProvider * ) CAccessor::Access( AID_STRINGSPROVIDER );

	( ( wxStaticText * ) FindWindowById( ID_ST_PROFILESPAGE_PRV_OPMODE_VAL ) )->SetLabel( sp->OperModeStrings[ dev_data->GetOperatingMode( ) ] );

	( ( wxStaticText * ) FindWindowById( ID_ST_PROFILESPAGE_PRV_CHANNEL_VAL ) )->SetLabel( wxString::Format( _( "%d" ), dev_data->GetChannel( ) ) );

	wxString ssid;

	for( int c = 0; c < dev_data->GetSSIDLength( ); c++ )
	{
		ssid.Append( dev_data->GetSSID( )[ c ] );
	}

	( ( wxStaticText * ) FindWindowById( ID_ST_PROFILESPAGE_PRV_SSID_VAL ) )->SetLabel( ssid );
	( ( wxStaticText * ) FindWindowById( ID_ST_PROFILESPAGE_PRV_RADIO_VAL ) )->SetLabel( sp->RadioStrings[ dev_data->GetRadioIsOn( ) ] );
	( ( wxStaticText * ) FindWindowById( ID_ST_PROFILESPAGE_PRV_INTROAMING_VAL ) )->SetLabel( sp->RoamingStrings[ dev_data->GetInternationalRoaming( ) ] );

	( ( wxStaticText * ) FindWindowById( ID_ST_PROFILESPAGE_PRV_TRRATE_VAL ) )->SetLabel( sp->TransmitRateStrings[ dev_data->GetTxRate( ) ] );
	( ( wxStaticText * ) FindWindowById( ID_ST_PROFILESPAGE_PRV_PRTYPE_VAL ) )->SetLabel( sp->PreambleTypeStrings[ dev_data->GetPreambleType( ) ] );
	( ( wxStaticText * ) FindWindowById( ID_ST_PROFILESPAGE_PRV_PWRMODE_VAL ) )->SetLabel( sp->PowerModeStrings[ dev_data->GetPowerMgmtMode( ) ] );

	( ( wxStaticText * ) FindWindowById( ID_ST_PROFILESPAGE_PRV_FRAGTHRSHLD_VAL ) )->SetLabel( wxString::Format( wxT( "%d" ), dev_data->GetFragmentationThreshold( ) ) );
	( ( wxStaticText * ) FindWindowById( ID_ST_PROFILESPAGE_PRV_RTSCTSTHR_VAL ) )->SetLabel( wxString::Format( wxT( "%d" ), dev_data->GetRtsCtsThreshold( ) ) );

	int encryption_level = wep_data->GetEncryptionLevel( );

	( ( wxStaticText * ) FindWindowById( ID_ST_PROFILESPAGE_PRV_ENCRYPTION_VAL ) )->SetLabel( sp->EncryptionStrings[ wep_data->GetEncryptionLevel( ) ] );

	if( encryption_level == 0 )
	{
		( ( wxStaticText * ) FindWindowById( ID_ST_PROFILESPAGE_PRV_WEPKEYTOUSE ) )->SetLabel( wxT( "" ) );
		( ( wxStaticText * ) FindWindowById( ID_ST_PROFILESPAGE_PRV_WEPKEYTOUSE_VAL ) )->SetLabel( wxT( "" ) );

		( ( wxStaticText * ) FindWindowById( ID_ST_PROFILESPAGE_PRV_WEPMODE ) )->SetLabel( wxT( "" ) );
		( ( wxStaticText * ) FindWindowById( ID_ST_PROFILESPAGE_PRV_WEPMODE_VAL ) )->SetLabel( wxT( "" ) );

		( ( wxStaticText * ) FindWindowById( ID_ST_PROFILESPAGE_PRV_AUTHENTICATIONTYPE ) )->SetLabel( wxT( "" ) );
		( ( wxStaticText * ) FindWindowById( ID_ST_PROFILESPAGE_PRV_AUTHENTICATIONTYPE_VAL ) )->SetLabel( wxT( "" ) );

		( ( wxStaticText * ) FindWindowById( ID_ST_PROFILESPAGE_PRV_WEPNOTSHOWN ) )->SetLabel( wxT( "" ) );
	}
	else
	{
		( ( wxStaticText * ) FindWindowById( ID_ST_PROFILESPAGE_PRV_WEPKEYTOUSE ) )->SetLabel( _( "WepKeyToUse :" ) );
#ifdef ATMEL_WLAN
		( ( wxStaticText * ) FindWindowById( ID_ST_PROFILESPAGE_PRV_WEPKEYTOUSE_VAL ) )->SetLabel( sp->WepKeysStrings[ (int) ( wep_data->GetWepKeyToUse( ) ) ] );
#else
		( ( wxStaticText * ) FindWindowById( ID_ST_PROFILESPAGE_PRV_WEPKEYTOUSE_VAL ) )->SetLabel( sp->WepKeysStrings[ (int) ( wep_data->GetWepKeyToUse( ) - 1 ) ] );
#endif
		
		( ( wxStaticText * ) FindWindowById( ID_ST_PROFILESPAGE_PRV_WEPMODE ) )->SetLabel( _( "WepMode :" ) );
		( ( wxStaticText * ) FindWindowById( ID_ST_PROFILESPAGE_PRV_WEPMODE_VAL ) )->SetLabel( sp->WepModeStrings[ (int) ( wep_data->GetWepMode( ) ) ] );
		
		( ( wxStaticText * ) FindWindowById( ID_ST_PROFILESPAGE_PRV_AUTHENTICATIONTYPE ) )->SetLabel( _( "AuthenticationType :" ) );
		( ( wxStaticText * ) FindWindowById( ID_ST_PROFILESPAGE_PRV_AUTHENTICATIONTYPE_VAL ) )->SetLabel( sp->AuthTypeStrings[ (int) ( wep_data->GetAuthenticationType( ) ) ] );

		( ( wxStaticText * ) FindWindowById( ID_ST_PROFILESPAGE_PRV_WEPNOTSHOWN ) )->SetLabel( _( "WEP keys are NOT shown for security reasons." ) );
	}

	delete model;
	return;
}

void CPreviewPanel::ClearPreview( )
{
	( ( wxStaticText * ) FindWindowById( ID_ST_PROFILESPAGE_PRV_OPMODE_VAL ) )->SetLabel( wxT( "" ) );
	( ( wxStaticText * ) FindWindowById( ID_ST_PROFILESPAGE_PRV_CHANNEL_VAL ) )->SetLabel( wxT( "" ) );
	( ( wxStaticText * ) FindWindowById( ID_ST_PROFILESPAGE_PRV_SSID_VAL ) )->SetLabel( wxT( "" ) );
	( ( wxStaticText * ) FindWindowById( ID_ST_PROFILESPAGE_PRV_RADIO_VAL ) )->SetLabel( wxT( "" ) );
	( ( wxStaticText * ) FindWindowById( ID_ST_PROFILESPAGE_PRV_INTROAMING_VAL ) )->SetLabel( wxT( "" ) );
	( ( wxStaticText * ) FindWindowById( ID_ST_PROFILESPAGE_PRV_TRRATE_VAL ) )->SetLabel( wxT( "" ) );
	( ( wxStaticText * ) FindWindowById( ID_ST_PROFILESPAGE_PRV_PRTYPE_VAL ) )->SetLabel( wxT( "" ) );
	( ( wxStaticText * ) FindWindowById( ID_ST_PROFILESPAGE_PRV_PWRMODE_VAL ) )->SetLabel( wxT( "" ) );
	( ( wxStaticText * ) FindWindowById( ID_ST_PROFILESPAGE_PRV_FRAGTHRSHLD_VAL ) )->SetLabel( wxT( "" ) );
	( ( wxStaticText * ) FindWindowById( ID_ST_PROFILESPAGE_PRV_RTSCTSTHR_VAL ) )->SetLabel( wxT( "" ) );
	( ( wxStaticText * ) FindWindowById( ID_ST_PROFILESPAGE_PRV_ENCRYPTION_VAL ) )->SetLabel( wxT( "" ) );
	( ( wxStaticText * ) FindWindowById( ID_ST_PROFILESPAGE_PRV_WEPKEYTOUSE_VAL ) )->SetLabel( wxT( "" ) );
	( ( wxStaticText * ) FindWindowById( ID_ST_PROFILESPAGE_PRV_WEPMODE_VAL ) )->SetLabel( wxT( "" ) );
	( ( wxStaticText * ) FindWindowById( ID_ST_PROFILESPAGE_PRV_AUTHENTICATIONTYPE_VAL ) )->SetLabel( wxT( "" ) );
	( ( wxStaticText * ) FindWindowById( ID_ST_PROFILESPAGE_PRV_WEPNOTSHOWN ) )->SetLabel( wxT( "" ) );

	return;
}


