/* WinterData.cpp */
/*******************************************************************************************
	Copyright 2002-2003 ATMEL Corporation.

	This file is part of ATMEL Wireless LAN Drivers.

	ATMEL Wireless LAN Drivers is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	ATMEL Wireless LAN Drivers is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with ATMEL Wireless LAN Drivers; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*******************************************************************************************/

#include "WinterData.h"


// CWinterData class implementation
//==================================
CWinterData::CWinterData( CWinterModel *model ) : CWinterModelChild( model )
{
}

CWinterData::~CWinterData( )
{
}


// CMacAddressData class implementation
//======================================
CMacAddressData::CMacAddressData( CWinterModel *model, UCHAR *mac_address ) : CWinterData( model  )
{
}

CMacAddressData::~CMacAddressData( )
{
}

bool CMacAddressData::Import( void *from )
{
	SetMacAddress( ( UCHAR * ) from );
	return true;
}

bool CMacAddressData::Export( void *to )
{
	// This function will not be used so far.
	memcpy( to, GetMacAddress( ), 6 );
	return true;
}

void CMacAddressData::SetMacAddress( UCHAR mac[MAC_LENGTH] )
{
	memcpy( mMacAddress, mac, MAC_LENGTH );
	return;
}

UCHAR *CMacAddressData::GetMacAddress( )
{
	return mMacAddress;
}


// CWepInfoData class implementation
//===================================
CWepInfoData::CWepInfoData( CWinterModel *model, WEP_INFO *src ) : CWinterData( model )
{
}

CWepInfoData::~CWepInfoData( )
{

}

bool CWepInfoData::doimport( WEP_INFO *src )
{
	if( src == NULL )
	{
		return false;
	}

	SetWepKeyToUse( src->WepKeyToUse );
	SetWepMode( src->WepMode );
	SetAuthenticationType( src->AuthenticationType );
	SetEncryptionLevel( src->EncryptionLevel );
#ifdef ATMEL_WLAN
	for(int i=0; i<4; i++)
	{
		SetWepKey( src->KeyMaterial[i], i );
	}
#else
	SetWepKey1( src->WepKey1 );
	SetWepKey2( src->WepKey2 );
	SetWepKey3( src->WepKey3 );
	SetWepKey4( src->WepKey4 );
#endif

	SetModified( true );
	return true;
}

bool CWepInfoData::doexport( WEP_INFO *tgt )
{
	tgt->WepKeyToUse = GetWepKeyToUse( );
	tgt->WepMode = GetWepMode( );
	tgt->AuthenticationType = GetAuthenticationType( );
	tgt->EncryptionLevel = GetEncryptionLevel( );
#ifdef ATMEL_WLAN
	switch(tgt->EncryptionLevel)
	{
		case 0 :
			tgt->KeyLength =0;
			tgt->PairwiseCipherSuite = CIPHER_SUITE_NONE;
			break;
		case 1 :
			tgt->KeyLength =5;
			tgt->PairwiseCipherSuite = CIPHER_SUITE_WEP_64;
			break;
		case 2 :
			tgt->KeyLength =13;
			tgt->PairwiseCipherSuite = CIPHER_SUITE_WEP_128;
			break;
	}
	for(int i=0; i<4; i++)
	{
		memcpy( tgt->KeyMaterial[i], GetWepKey(i), LONG_WEP_KEY_SIZE );
	}
	tgt->KeyMaterial[tgt->WepKeyToUse][39] = 1;
		
#else
	memcpy( tgt->WepKey1, GetWepKey1(), LONG_WEP_KEY_SIZE );
	memcpy( tgt->WepKey2, GetWepKey2(), LONG_WEP_KEY_SIZE );
	memcpy( tgt->WepKey3, GetWepKey3(), LONG_WEP_KEY_SIZE );
	memcpy( tgt->WepKey4, GetWepKey4(), LONG_WEP_KEY_SIZE );
#endif

	return true;
}

bool CWepInfoData::Import( void *from )
{
	return doimport( ( WEP_INFO * ) from );
}

bool CWepInfoData::Export( void *to )
{
	return doexport( ( WEP_INFO * ) to );
}

void CWepInfoData::SetWepKeyToUse( UCHAR wepkey )
{
	mWepKeyToUse = wepkey;
	SetModified( true );
	return;
}

UCHAR CWepInfoData::GetWepKeyToUse( )
{
	return mWepKeyToUse;
}

void CWepInfoData::SetWepMode( UCHAR wepmode )
{
	mWepMode = wepmode;
	SetModified( true );
	return;
}

UCHAR CWepInfoData::GetWepMode( )
{
	return mWepMode;
}

void CWepInfoData::SetAuthenticationType( USHORT authtype )
{
	mAuthenticationType = authtype;
	SetModified( true );
	return;
}

USHORT CWepInfoData::GetAuthenticationType( )
{
	return mAuthenticationType;
}

void CWepInfoData::SetEncryptionLevel( int encryptlevel )
{
	mEncryptionLevel = encryptlevel;
	SetModified( true );
	return;
}

int CWepInfoData::GetEncryptionLevel( )
{
	return mEncryptionLevel;
}

#ifdef ATMEL_WLAN

UCHAR *CWepInfoData::GetWepKey( int n )
{	
	return mWepKey[n];
}

void CWepInfoData::SetWepKey( UCHAR wepkey[LONG_WEP_KEY_SIZE], int n )
{
	memcpy( mWepKey[n], wepkey, LONG_WEP_KEY_SIZE );
	SetModified( true );
	return;
}

#else

void CWepInfoData::SetWepKey1( UCHAR wepkey1[ LONG_WEP_KEY_SIZE ] )
{
	memcpy( mWepKey1, wepkey1, LONG_WEP_KEY_SIZE );
	SetModified( true );
	return;
}

UCHAR *CWepInfoData::GetWepKey1( )
{
	return mWepKey1;
}

void CWepInfoData::SetWepKey2( UCHAR wepkey2[ LONG_WEP_KEY_SIZE ] )
{
	memcpy( mWepKey2, wepkey2, LONG_WEP_KEY_SIZE );
	SetModified( true );
	return;
}

UCHAR *CWepInfoData::GetWepKey2( )
{
	return mWepKey2;
}

void CWepInfoData::SetWepKey3( UCHAR wepkey3[ LONG_WEP_KEY_SIZE ] )
{
	memcpy( mWepKey3, wepkey3, LONG_WEP_KEY_SIZE );
	SetModified( true );
	return;
}

UCHAR *CWepInfoData::GetWepKey3( )
{
	return mWepKey3;
}

void CWepInfoData::SetWepKey4( UCHAR wepkey4[ LONG_WEP_KEY_SIZE ] )
{
	memcpy( mWepKey4, wepkey4, LONG_WEP_KEY_SIZE );
	SetModified( true );
	return;
}

UCHAR *CWepInfoData::GetWepKey4( )
{
	return mWepKey4;
}

#endif


// CBssInfoData class implementation
//===================================
CBssInfoData::CBssInfoData( BSS_INFO *src )
{
	if( src != NULL )
	{
		doimport( src );
	}
}

CBssInfoData::~CBssInfoData( )
{

}

bool CBssInfoData::doimport( BSS_INFO *src )
{
	if( src == NULL )
	{
		return false;
	}

	SetChannel( src->Channel );
	SetSSID( src->SSID );
	SetSSIDsize( src->SSIDsize );
	SetBSSID( src->BSSID );
	SetRSSI( src->RSSI );
	SetUsingWep( src->UsingWEP );
	SetPreambleType( src->PreambleType );
	SetBeaconPeriod( src->BeaconPeriod );
	SetBSStype( src->BSStype );

	return true;
}

bool CBssInfoData::doexport( BSS_INFO *tgt )
{
	tgt->Channel = GetChannel( );

	memcpy( tgt->SSID, GetSSID( ), MAX_SSID_LENGTH );
	memcpy( tgt->BSSID, GetBSSID( ), 6 );

	tgt->SSIDsize = GetSSIDsize( );
	tgt->RSSI = GetRSSI( );
	tgt->UsingWEP = GetUsingWep( );
	tgt->PreambleType = GetPreambleType( );
	tgt->BeaconPeriod = GetBeaconPeriod( );
	tgt->BSStype = GetBSStype( );

	return true;
}

bool CBssInfoData::Import( void *from )
{
	return doimport( ( BSS_INFO * ) from );
}

bool CBssInfoData::Export( void *to )
{
	return doexport( ( BSS_INFO * ) to );
}

void CBssInfoData::SetChannel( int channel )
{
	mChannel = channel;
	return;
}

int CBssInfoData::GetChannel( )
{
	return mChannel;
}

void CBssInfoData::SetSSID(  UCHAR ssid[ MAX_SSID_LENGTH ] )
{
	memcpy( mSSID, ssid, MAX_SSID_LENGTH );
	return;
}

UCHAR *CBssInfoData::GetSSID( )
{
	return mSSID;
}

void CBssInfoData::SetSSIDsize( UCHAR ssidsize )
{
	mSSIDsize = ssidsize;
	return;
}

UCHAR CBssInfoData::GetSSIDsize( )
{
	return mSSIDsize;
}

void CBssInfoData::SetBSSID( UCHAR bssid[6] )
{
	memcpy( mBSSID, bssid, 6 );
	return;
}

UCHAR *CBssInfoData::GetBSSID( )
{
	return mBSSID;
}

void CBssInfoData::SetRSSI( UCHAR rssi )
{
	mRSSI = rssi;
	return;
}

UCHAR CBssInfoData::GetRSSI( )
{
	return mRSSI;
}

void CBssInfoData::SetUsingWep( UCHAR usingwep )
{
	mUsingWep = usingwep;
	return;
}

UCHAR CBssInfoData::GetUsingWep( )
{
	return mUsingWep;
}

void CBssInfoData::SetPreambleType( UCHAR preambletype )
{
	mPreambleType = preambletype;
	return;
}

UCHAR CBssInfoData::GetPreambleType( )
{
	return mPreambleType;
}

void CBssInfoData::SetBeaconPeriod( USHORT beaconperiod )
{
	mBeaconPeriod = beaconperiod;
	return;
}

USHORT CBssInfoData::GetBeaconPeriod( )
{
	return mBeaconPeriod;
}

void CBssInfoData::SetBSStype( UCHAR bsstype )
{
	mBSStype = bsstype;
	return;
}

UCHAR CBssInfoData::GetBSStype( )
{
	return mBSStype;
}


// CVersionInfoData class implementation
//=======================================
CVersionInfoData::CVersionInfoData( CWinterModel *model, VERSION_INFO *src ) : CWinterData( model )
{

}

CVersionInfoData::~CVersionInfoData( )
{

}

bool CVersionInfoData::doimport( VERSION_INFO *src )
{
	if( src == NULL )
	{
		return false;
	}

	SetDriverMajorVersion( src->DriverMajorVersion );
	SetDriverMinorVersion( src->DriverMinorVersion );
	SetDriverSubVersion( src->DriverSubVersion );
	SetDriverBuild( src->DriverBuild );
	SetFwMajorVersion( src->FwMajorVersion );
	SetFwMinorVersion( src->FwMinorVersion );
	SetFwSubVersion( src->FwSubVersion );
	SetFwBuild( src->FwBuild );

	SetModified( true );
	return true;
}

bool CVersionInfoData::doexport( VERSION_INFO *tgt )
{
	tgt->DriverMajorVersion = GetDriverMajorVersion( );
	tgt->DriverMinorVersion = GetDriverMinorVersion( );
	tgt->DriverSubVersion = GetDriverSubVersion( );
	tgt->DriverBuild = GetDriverBuild( );
	tgt->FwMajorVersion = GetFwMajorVersion( );
	tgt->FwMinorVersion = GetFwMinorVersion( );
	tgt->FwSubVersion = GetFwSubVersion( );
	tgt->FwBuild = GetFwBuild( );


	return true;
}

bool CVersionInfoData::Import( void *from )
{
	return doimport( ( VERSION_INFO * ) from );
}

bool CVersionInfoData::Export( void *to )
{
	return doexport( ( VERSION_INFO * ) to );
}

void CVersionInfoData::SetDriverMajorVersion( UCHAR majorversion )
{
	mDriverMajorVersion = majorversion;
	SetModified( true );
	return;
}

UCHAR CVersionInfoData::GetDriverMajorVersion( )
{
	return mDriverMajorVersion;
}

void CVersionInfoData::SetDriverMinorVersion( UCHAR minorversion )
{
	mDriverMinorVersion = minorversion;
	SetModified( true );
	return;
}

UCHAR CVersionInfoData::GetDriverMinorVersion( )
{
	return mDriverMinorVersion;
}

void CVersionInfoData::SetDriverSubVersion( UCHAR subversion )
{
	mDriverSubVersion = subversion;
	SetModified( true );
	return;
}

UCHAR CVersionInfoData::GetDriverSubVersion( )
{
	return mDriverSubVersion;
}

void CVersionInfoData::SetDriverBuild( USHORT driverbuild )
{
	mDriverBuild = driverbuild;
	SetModified( true );
	return;
}

USHORT CVersionInfoData::GetDriverBuild( )
{
	return mDriverBuild;
}

void CVersionInfoData::SetFwMajorVersion( USHORT fwmajorversion )
{
	mFwMajorVersion = fwmajorversion;
	SetModified( true );
	return;
}

USHORT CVersionInfoData::GetFwMajorVersion( )
{
	return mFwMajorVersion;
}

void CVersionInfoData::SetFwMinorVersion( USHORT fwminorversion )
{
	mFwMinorVersion = fwminorversion;
	SetModified( true );
	return;

}

USHORT CVersionInfoData::GetFwMinorVersion( )
{
	return mFwMinorVersion;
}

void CVersionInfoData::SetFwSubVersion( USHORT fwsubversion )
{
	mFwSubVersion = fwsubversion;
	SetModified( true );
	return;
}

USHORT CVersionInfoData::GetFwSubVersion( )
{
	return mFwSubVersion;
}

void CVersionInfoData::SetFwBuild( USHORT fwbuild )
{
	mFwBuild = fwbuild;
	SetModified( true );
	return;
}

USHORT CVersionInfoData::GetFwBuild( )
{
	return mFwBuild;
}


// CStatisticsData class implementation
//======================================
CStatisticsData::CStatisticsData( CWinterModel *model, STATISTICS *src ) : CWinterData( model )
{
}

CStatisticsData::~CStatisticsData( )
{

}

bool CStatisticsData::doimport( STATISTICS *src )
{
	if( src == NULL )
	{
		return false;
	}

	SetTxDataPacketsOk( src->TxDataPacketsOk );
	SetTxDataPacketsError( src->TxDataPacketsError );
	SetTxMgmtPacketsOk( src->TxMgmtPacketsOk );
	SetTxMgmtPacketsError( src->TxMgmtPacketsError );
	SetRxDataPacketsOk( src->RxDataPacketsOk );
	SetRxDataPacketsError( src->RxDataPacketsError );
	SetRxMgmtPacketsOk( src->RxMgmtPacketsOk );
	SetRxMgmtPacketsError( src->RxMgmtPacketsError );
	SetRxLost( src->RxLost );
	SetTxPacketsRejectedNotReady( src->TxPacketsRejectedNotReady );
	SetTxPacketsRejectedResources( src->TxPacketsRejectedResources );
	SetMatchingBeacons( src->MatchingBeacons );

	SetModified( true );
	return true;
}

bool CStatisticsData::doexport( STATISTICS *tgt )
{
	tgt->TxDataPacketsOk = GetTxDataPacketsOk( );
	tgt->TxDataPacketsError = GetTxDataPacketsError( );
	tgt->TxMgmtPacketsOk = GetTxMgmtPacketsOk( );
	tgt->TxMgmtPacketsError = GetTxMgmtPacketsError( );
	tgt->RxDataPacketsOk = GetRxDataPacketsOk( );
	tgt->RxDataPacketsError = GetRxDataPacketsError( );
	tgt->RxMgmtPacketsOk = GetRxMgmtPacketsOk( );
	tgt->RxMgmtPacketsError = GetRxMgmtPacketsError( );
	tgt->RxLost = GetRxLost( );
	tgt->TxPacketsRejectedNotReady = GetTxPacketsRejectedNotReady( );
	tgt->TxPacketsRejectedResources = GetTxPacketsRejectedResources( );
	tgt->MatchingBeacons = GetMatchingBeacons( );

	return true;
}

bool CStatisticsData::Import( void *from )
{
	return doimport( ( STATISTICS * ) from );
}

bool CStatisticsData::Export( void *to )
{
	return doexport( ( STATISTICS * ) to );
}

void CStatisticsData::SetTxDataPacketsOk( ULONG txdatapacketsok )
{
	mTxDataPacketsOk = txdatapacketsok;
	SetModified( true );
	return;
}

ULONG CStatisticsData::GetTxDataPacketsOk( )
{
	return mTxDataPacketsOk;
}

void CStatisticsData::SetTxDataPacketsError( ULONG txdatapacketserror )
{
	mTxDataPacketsError = txdatapacketserror;
	SetModified( true );
	return;
}

ULONG CStatisticsData::GetTxDataPacketsError( )
{
	return mTxDataPacketsError;
}

void CStatisticsData::SetTxMgmtPacketsOk( ULONG txmgmtpacketsok )
{
	mTxMgmtPacketsOk = txmgmtpacketsok;
	SetModified( true );
	return;
}

ULONG CStatisticsData::GetTxMgmtPacketsOk( )
{
	return mTxMgmtPacketsOk;
}

void CStatisticsData::SetTxMgmtPacketsError( ULONG txmgmtpacketserror )
{
	mTxMgmtPacketsError = txmgmtpacketserror;
	SetModified( true );
	return;
}

ULONG CStatisticsData::GetTxMgmtPacketsError( )
{
	return mTxMgmtPacketsError;
}

void CStatisticsData::SetRxDataPacketsOk( ULONG rxdatapacketsok )
{
	mRxDataPacketsOk = rxdatapacketsok;
	SetModified( true );
	return;
}

ULONG CStatisticsData::GetRxDataPacketsOk( )
{
	return mRxDataPacketsOk;
}

void CStatisticsData::SetRxDataPacketsError( ULONG rxdatapacketserror )
{
	mRxDataPacketsError = rxdatapacketserror;
	SetModified( true );
	return;
}

ULONG CStatisticsData::GetRxDataPacketsError( )
{
	return mRxDataPacketsError;
}

void CStatisticsData::SetRxMgmtPacketsOk( ULONG rxmgmtpacketsok )
{
	mRxMgmtPacketsOk = rxmgmtpacketsok;
	SetModified( true );
	return;
}

ULONG CStatisticsData::GetRxMgmtPacketsOk( )
{
	return mRxMgmtPacketsOk;
}

void CStatisticsData::SetRxMgmtPacketsError( ULONG rxmgmtpacketserror )
{
	mRxMgmtPacketsError = rxmgmtpacketserror; 
	SetModified( true );
	return;
}

ULONG CStatisticsData::GetRxMgmtPacketsError( )
{
	return mRxMgmtPacketsError;
}

void CStatisticsData::SetRxLost( ULONG rxlost )
{
	mRxLost = rxlost;
	SetModified( true );
	return;
}

ULONG CStatisticsData::GetRxLost( )
{
	return mRxLost;
}

void CStatisticsData::SetTxPacketsRejectedNotReady( ULONG txpacketsrejnotready )
{
	mTxPacketsRejectedNotReady = txpacketsrejnotready;
	SetModified( true );
	return;
}

ULONG CStatisticsData::GetTxPacketsRejectedNotReady( )
{
	return mTxPacketsRejectedNotReady;
}

void CStatisticsData::SetTxPacketsRejectedResources( ULONG txpacketsrejresources )
{
	mTxPacketsRejectedResources = txpacketsrejresources;
	SetModified( true );
	return;
}

ULONG CStatisticsData::GetTxPacketsRejectedResources( )
{
	return mTxPacketsRejectedResources;
}

void CStatisticsData::SetMatchingBeacons( ULONG matchingbeacons )
{
	mMatchingBeacons = matchingbeacons;
	SetModified( true );
	return;
}

ULONG CStatisticsData::GetMatchingBeacons( )
{
	return mMatchingBeacons;
}


// CDeviceConfigurationData class implementation
//===============================================
CDeviceConfigurationData::CDeviceConfigurationData( CWinterModel *model, DEVICE_CONFIGURATION *src ) : CWinterData( model )
{
}

CDeviceConfigurationData::~CDeviceConfigurationData( )
{

}

bool CDeviceConfigurationData::doimport( DEVICE_CONFIGURATION *src )
{
	if( src == NULL )
	{
		return false;
	}

	SetOperatingMode( src->OperatingMode );
	SetChannel( src->Channel );
	SetSSID( src->SSID );
	SetSSIDLength( src->SSIDlength );
	SetTxRate( src->TxRate );
	SetPowerMgmtMode( src->PowerMgmtMode );
	SetInternationalRoaming( src->InternationalRoaming );
	SetRadioIsOn( src->RadioIsOn );
	SetPreambleType( src->PreambleType );
	SetFragmentationThreshold( src->FragmentationThreshold );
	SetRtsCtsThreshold( src->RtsCtsThreshold );
	SetBSSID( src->BSSID );
	SetStationState( src->StationState );
	SetRssi( src->Rssi );
	SetLinkQuality( src->LinkQuality );
	SetRegDomain( src->RegDomain );
	SetChannelVector( src->ChannelVector );
	SetMgmtErrorCode( src->MgmtErrorCode );

	SetModified( true );
	return true;
}

bool CDeviceConfigurationData::doexport( DEVICE_CONFIGURATION *tgt )
{
	tgt->OperatingMode = GetOperatingMode( );
	tgt->Channel = GetChannel( );

	memcpy( tgt->SSID, GetSSID( ), MAX_SSID_LENGTH );
	tgt->SSIDlength = GetSSIDLength( );
	memcpy( tgt->BSSID, GetBSSID( ), 6 );
	memcpy( tgt->ChannelVector, GetChannelVector( ), 14 );

	tgt->TxRate = GetTxRate( );
	tgt->PowerMgmtMode = GetPowerMgmtMode( );
	tgt->InternationalRoaming = GetInternationalRoaming( );
	tgt->RadioIsOn = GetRadioIsOn( );
	tgt->PreambleType = GetPreambleType( );
	tgt->FragmentationThreshold = GetFragmentationThreshold( );
	tgt->RtsCtsThreshold = GetRtsCtsThreshold( );
	tgt->StationState = GetStationState( );
	tgt->Rssi = GetRssi( );
	tgt->LinkQuality = GetLinkQuality( );
	tgt->RegDomain = GetRegDomain( );
	tgt->MgmtErrorCode = GetMgmtErrorCode( );

	return true;
}

bool CDeviceConfigurationData::Import( void *from )
{
	return doimport( ( DEVICE_CONFIGURATION * ) from );
}

bool CDeviceConfigurationData::Export( void *to )
{
	return doexport( ( DEVICE_CONFIGURATION * ) to );
}

void CDeviceConfigurationData::SetOperatingMode( UCHAR opermode )
{
	mOperatingMode = opermode;
	SetModified( true );
	return;
}

UCHAR CDeviceConfigurationData::GetOperatingMode( )
{
	return mOperatingMode;
}

void CDeviceConfigurationData::SetChannel( UCHAR channel )
{
	mChannel = channel;
	SetModified( true );
	return;
}

UCHAR CDeviceConfigurationData::GetChannel( )
{
	return mChannel;
}


void CDeviceConfigurationData::SetSSID( UCHAR ssid[ MAX_SSID_LENGTH ] )
{
	memcpy( mSSID, ssid, MAX_SSID_LENGTH );
	SetModified( true );
	return;
}

UCHAR *CDeviceConfigurationData::GetSSID( )
{
	return mSSID;
}

void CDeviceConfigurationData::SetSSIDLength( UCHAR ssid_length )
{
	mSSIDLength = ssid_length;
	return;
}

UCHAR CDeviceConfigurationData::GetSSIDLength( )
{
	return mSSIDLength;
}

void CDeviceConfigurationData::SetTxRate( UCHAR txrate )
{
	mTxRate = txrate;
	SetModified( true );
	return;
}

UCHAR CDeviceConfigurationData::GetTxRate( )
{
	return mTxRate;
}

void CDeviceConfigurationData::SetPowerMgmtMode( UCHAR powermgmtmode )
{
	mPowerMgmtMode = powermgmtmode;
	SetModified( true );
	return;
}

UCHAR CDeviceConfigurationData::GetPowerMgmtMode( )
{
	return mPowerMgmtMode;
}

void CDeviceConfigurationData::SetInternationalRoaming( UCHAR introaming )
{
	mInternationalRoaming = introaming;
	SetModified( true );
	return;
}

UCHAR CDeviceConfigurationData::GetInternationalRoaming( )
{
	return mInternationalRoaming;
}

void CDeviceConfigurationData::SetRadioIsOn( UCHAR radioison )
{
	mRadioIsOn = radioison;
	SetModified( true );
	return;
}

UCHAR CDeviceConfigurationData::GetRadioIsOn( )
{
	return mRadioIsOn;
}

void CDeviceConfigurationData::SetPreambleType( UCHAR preambletype )
{
	mPreambleType = preambletype;
	SetModified( true );
	return;
}

UCHAR CDeviceConfigurationData::GetPreambleType( )
{
	return mPreambleType;
}

void CDeviceConfigurationData::SetFragmentationThreshold( USHORT fragthreshold )
{
	mFragmentationThreshold = fragthreshold;
	SetModified( true );
	return;
}

USHORT CDeviceConfigurationData::GetFragmentationThreshold( )
{
	return mFragmentationThreshold;
}

void CDeviceConfigurationData::SetRtsCtsThreshold( USHORT rtsctsthreshold )
{
	mRtsCtsThreshold = rtsctsthreshold;
	SetModified( true );
	return;
}

USHORT CDeviceConfigurationData::GetRtsCtsThreshold( )
{
	return mRtsCtsThreshold;
}

void CDeviceConfigurationData::SetBSSID( UCHAR bssid[6] )
{
	memcpy( mBSSID, bssid, 6 );
	SetModified( true );
	return;
}

UCHAR *CDeviceConfigurationData::GetBSSID( )
{
	return mBSSID;
}

void CDeviceConfigurationData::SetStationState( UCHAR stationstate )
{
	mStationState = stationstate;
	SetModified( true );
	return;
}

UCHAR CDeviceConfigurationData::GetStationState( )
{
	return mStationState;
}

void CDeviceConfigurationData::SetRssi( UCHAR rssi )
{
	mRssi = rssi;
	SetModified( true );
	return;
}

UCHAR CDeviceConfigurationData::GetRssi( )
{
	return mRssi;
}

void CDeviceConfigurationData::SetLinkQuality( UCHAR linkquality )
{
	mLinkQuality = linkquality;
	SetModified( true );
	return;
}

UCHAR CDeviceConfigurationData::GetLinkQuality( )
{
	return mLinkQuality;
}

void CDeviceConfigurationData::SetRegDomain( UCHAR regdomain )
{
	mRegDomain = regdomain;
	SetModified( true );
	return;
}

UCHAR CDeviceConfigurationData::GetRegDomain( )
{
	return mRegDomain;
}

void CDeviceConfigurationData::SetChannelVector( UCHAR channelvector[14] )
{
	memcpy( mChannelVector, channelvector, 14 );
	SetModified( true );
	return;
}

UCHAR *CDeviceConfigurationData::GetChannelVector( )
{
	return mChannelVector;
}

void CDeviceConfigurationData::SetMgmtErrorCode( ULONG mgmterrorcode )
{
	mMgmtErrorCode = mgmterrorcode;
	SetModified( true );
	return;
}

ULONG CDeviceConfigurationData::GetMgmtErrorCode( )
{
	return mMgmtErrorCode;
}


