/* WinterModel.cpp */
/*******************************************************************************************
	Copyright 2002-2003 ATMEL Corporation.

	This file is part of ATMEL Wireless LAN Drivers.

	ATMEL Wireless LAN Drivers is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	ATMEL Wireless LAN Drivers is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with ATMEL Wireless LAN Drivers; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*******************************************************************************************/

#include "WinterModel.h"
#include "WinterData.h"


// CWinterModel class implementation
//===================================
CWinterModel::CWinterModel( const wxString &name )
{
	mName = name;
	new CWepInfoData( this, NULL );
	//new CBssInfoData( this, NULL );
	//new CConWepInfoData( this, NULL );
	new CVersionInfoData( this, NULL );
	//new CSupportInfoData( this, NULL );
	new CStatisticsData( this, NULL );
	//new CStatInfoData( this, NULL );
	new CDeviceConfigurationData( this, NULL );
	new CMacAddressData( this, NULL );
}

CWinterModel::~CWinterModel( )
{
	// Two step destruction :
	// 1. Get all the pointers to the children.
	// CMonitorStruct *m = ( CMonitorStruct * ) GetChild( MC_MONITOR );

	// 2. And then delete them.
	// delete m;

	CWinterModelChild *child;

	while( mChildren.GetCount( ) != 0 )
	{
		child = GetChild( 0 );
		delete child;
		RemoveChild( child );
	}
}

CWinterModelChild *CWinterModel::GetChild( int child )
{
	if( child == -1 )
	{
		return NULL;
	}

	return mChildren[ child ];
}

bool CWinterModel::IsModified( )
{
	bool ret = false;
	size_t count = mChildren.GetCount( );

	for( size_t c = 0; c < count; c++ )
	{
		ret = ret || ( mChildren[ c ] )->IsModified( );
	}

	return ret;
}

void CWinterModel::ResetModified( )
{
	size_t count = mChildren.GetCount( );

	for( size_t c = 0; c < count; c++ )
	{
		( mChildren[ c ] )->SetModified( false );
	}

	return;
}


bool CWinterModel::AddChild( CWinterModelChild *child )
{
	mChildren.Add( child );
	return true;
}

bool CWinterModel::RemoveChild( CWinterModelChild *child )
{
	int index = mChildren.Index( child );

	if( index != wxNOT_FOUND )
	{
		mChildren.RemoveAt( index );
		return true;
	}

	return false;
}


// CWinterModelChild class implementation
//========================================
CWinterModelChild::CWinterModelChild( CWinterModel *Model )
{
	mTheModel = Model;
	mIsModified = false;

	if( Model != NULL )
	{
		Model->AddChild( this );
	}
}

CWinterModelChild::~CWinterModelChild( )
{
	if( mTheModel != NULL )
	{
		mTheModel->RemoveChild( this );
	}
}

CWinterModel *CWinterModelChild::GetModel( )
{
	return mTheModel;
}

bool CWinterModelChild::IsModified( )
{
	return mIsModified;
}

void CWinterModelChild::SetModified( bool modified )
{
	mIsModified = modified;
	return;
}


